@@ -11,26 +11,19 @@
 #  PURPOSE.  See the GNU General Public License for more details.
 #
 
+import logging
 import sys
 
 import ninix.config
 
-def print_error(message):
-    sys.stderr.write(''.join((message, '\n')))
 
 def fatal(error):
-    print_error('alias.py: %s' % str(error))
+    logging.error('alias.py: {0}'.format(str(error)))
     return ninix.config.null_config()
 
 def create_from_file(path):
-    f = open(path)
-    buf = []
-    for line in f:
-        line = line.strip()
-        if not line:
-            continue
-        buf.append(line)
-    f.close()
+    with open(path) as f:
+        buf = [line.strip() for line in f if line.strip()]
     return create_from_buffer(buf)
 
 def create_from_buffer(buf):
@@ -82,7 +75,7 @@ def create_from_buffer(buf):
                             table[key].append(value)
                     else:
                         raise ValueError, 'malformed line found'
-            except ValueError, error:
+            except ValueError as error:
                 return fatal(error)
             dic[name] = table
         else:
@@ -100,6 +93,7 @@ def create_from_buffer(buf):
             dic[key] = value
     return dic
 
+
 # test
 if __name__ == '__main__':
     print create_from_file(sys.argv[1])
