@@ -407,7 +407,7 @@ class BalloonWindow(object):
     def scale(self):
         scalling = self.request_parent('GET', 'get_preference', 'balloon_scalling')
         scale = self.request_parent('GET', 'get_preference', 'surface_scale')
-        return scale if scalling else 100 # %
+        return scale if scalling else 100 # [%]
 
     def get_balloon_windowposition(self):
         winpos_x = self.__get_with_scaling('windowposition.x', int, 0)
