@@ -13,13 +13,13 @@
 #
 
 
-class Communicate:
+class Communicate(object):
 
     def __init__(self):
         self.__ghosts = {}
 
     def rebuild_ghostdb(self, sakura, name='', s0=0, s1=10):
-        if sakura in self.__ghosts.keys():
+        if sakura in self.__ghosts:
             del self.__ghosts[sakura]
         if name is None:
             return
@@ -27,26 +27,23 @@ class Communicate:
             self.__ghosts[sakura] = (name, s0, s1)
 
     def get_otherghostname(self, name):
-        otherghostname = []
-        for value in self.__ghosts.itervalues():
-            if value[0] != name:
-                otherghostname.append(chr(1).join(value))
-        return otherghostname
+        return [chr(1).join(value) for value in self.__ghosts.values() \
+                    if value[0] != name]
 
     def send_message(self, name, sender, sentence):
-        if name == "__SYSTEM_ALL_GHOST__":
-            for sakura in self.__ghosts.iterkeys():
+        if name == '__SYSTEM_ALL_GHOST__':
+            for sakura in self.__ghosts.keys():
                 sakura.enqueue_event('OnCommunicate', sender, sentence)
         elif chr(1) in name:
             to = name.split(chr(1))
-            for sakura, value in self.__ghosts.iteritems():
+            for sakura, value in self.__ghosts.items():
                 if value[0] in to:
                     sakura.enqueue_event('OnCommunicate', sender, sentence)
                     to.remove(value[0])
                     if not to:
                         break
         else:
-            for sakura, value in self.__ghosts.iteritems():
+            for sakura, value in self.__ghosts.items():
                 if value[0] == name:
                     sakura.enqueue_event('OnCommunicate', sender, sentence)
                     break
