@@ -19,22 +19,14 @@ import random
 import math
 import time
 import sys
+import logging
 
-if 'DISPLAY' in os.environ:
-    if 'gtk' not in sys.modules:
-        try:
-            import pygtk
-            pygtk.require('2.0')
-        except ImportError:
-            pass
-    import gtk
-    import glib
-    import pango
-    import cairo
-    import ninix.pix
-else:
-    gtk = None
+import gtk
+import glib
+import pango
+import cairo
 
+import ninix.pix
 import ninix.script
 from ninix.dll import SAORI
 
@@ -50,37 +42,41 @@ class Saori(SAORI):
         self.__sakura = sakura
 
     def check_import(self):
-        return 1 if self.__sakura and gtk else 0
+        return 1 if self.__sakura else 0
 
     def setup(self):
         self.blns = self.read_bln_txt(self.dir)
-        return 1 if self.blns else 0
+        if self.blns:
+            self.__sakura.attach_observer(self)
+            return 1
+        else:
+            return 0
 
     def read_bln_txt(self, dir):
         blns = {}
         try:
-            f = open(os.path.join(dir, 'bln.txt'), 'r')
-            data = {}
-            name = ''
-            for line in f:
-                line = line.strip()
-                if not line:
-                    continue
-                if line.startswith('//'):
-                    continue
-                if '[' in line:
-                    if name:
-                        blns[name] = (data, {})
-                    data = {}
-                    start = line.find('[')
-                    end = line.find(']')
-                    if end < 0:
-                        end = len(line)
-                    name = line[start + 1:end]
-                else:
-                    if ',' in line:
-                        key, value = [x.strip() for x in line.split(',', 1)]
-                        data[key] = value
+            with open(os.path.join(dir, 'bln.txt'), 'r') as f:
+                data = {}
+                name = ''
+                for line in f:
+                    line = line.strip()
+                    if not line:
+                        continue
+                    if line.startswith('//'):
+                        continue
+                    if '[' in line:
+                        if name:
+                            blns[name] = (data, {})
+                        data = {}
+                        start = line.find('[')
+                        end = line.find(']')
+                        if end < 0:
+                            end = len(line)
+                        name = line[start + 1:end]
+                    else:
+                        if ',' in line:
+                            key, value = [x.strip() for x in line.split(',', 1)]
+                            data[key] = value
             if name:
                 blns[name] = (data, {})
             return blns
@@ -95,8 +91,17 @@ class Saori(SAORI):
                     bln[bln_id].destroy()
                     del bln[bln_id]
         self.blns = {}
+        self.__sakura.detach_observer(self)
         return 1
 
+    def observer_update(self, event, args): ## FIXME
+        if event == 'set scale':
+            for name in self.blns:
+                data, bln = self.blns[name]
+                for bln_id in bln.keys():
+                    if bln[bln_id]:
+                        bln[bln_id].reset_scale()
+
     def execute(self, argument):
         if not argument:
             return self.RESPONSE[400]
@@ -144,7 +149,8 @@ class Saori(SAORI):
                     self.blns[name] = (data, {})
         return None
 
-class Balloon:
+
+class Balloon(object):
 
     def __init__(self, sakura, dir, data,
                  text, offset_x, offset_y, name, bln_id):
@@ -153,7 +159,12 @@ class Balloon:
         self.name = name
         self.id = bln_id
         self.data = data # XXX
-        self.window = ninix.pix.TransparentWindow(type=gtk.WINDOW_POPUP)
+        self.window = ninix.pix.TransparentWindow()
+        self.window.set_focus_on_map(False)
+        self.window.set_title(name)
+        self.window.set_skip_taskbar_hint(True)
+        self.window.set_decorated(False)
+        self.window.set_resizable(False)
         self.window.connect('delete_event', self.delete)
         self.position = data.get('position', 'sakura')
         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
@@ -182,38 +193,27 @@ class Balloon:
         except:
             self.destroy()
             return
-        self.scale = self.get_sakura_status('SurfaceScale') ## FIXME
+        self.balloon_pixbuf = balloon_pixbuf
         w = balloon_pixbuf.get_width()
         h = balloon_pixbuf.get_height()
-        w = max(8, int(w * self.scale / 100))
-        h = max(8, int(h * self.scale / 100))
-        balloon_pixbuf = balloon_pixbuf.scale_simple(
-            w, h, gtk.gdk.INTERP_BILINEAR)
-        mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
-        balloon_pixbuf.render_threshold_alpha(
-            mask_pixmap, 0, 0, 0, 0, w, h, 1)
-        self.x, self.y = self.get_coordinate(w, h)
+        self.x = self.y = 0
         if 'offset.x' in data:
-            self.x += self.direction * int(int(data['offset.x']) * self.scale / 100)
+            self.x += self.direction * int(data['offset.x'])
         if 'offset.y' in data:
-            self.y += int(int(data['offset.y']) * self.scale / 100)
+            self.y += int(data['offset.y'])
         if 'offset.random' in data:
-            self.x += int(int(data['offset.random']) * random.randrange(-1, 2) * self.scale / 100)
-            self.y += int(int(data['offset.random']) * random.randrange(-1, 2) * self.scale / 100)
-        self.x += self.direction * int(offset_x * self.scale / 100)
-        self.y += int(offset_y * self.scale / 100)
-        self.window.move(self.x, self.y)
-        gtk.gdk.flush() # XXX
-        self.left = int(int(data.get('disparea.left', 0)) * self.scale / 100)
-        self.right = int(int(data.get('disparea.right', w)) * self.scale / 100)
-        self.top = int(int(data.get('disparea.top', 0)) * self.scale / 100)
-        self.bottom = int(int(data.get('disparea.bottom', h)) * self.scale / 100)
-        if self.window.is_composited():
-            self.window.input_shape_combine_mask(mask_pixmap, 0, 0)
-        else:
-            self.window.shape_combine_mask(mask_pixmap, 0, 0)
-        self.current_mask = mask_pixmap
-        self.current_balloon_pixbuf = balloon_pixbuf
+            self.x += int(data['offset.random']) * random.randrange(-1, 2)
+            self.y += int(data['offset.random']) * random.randrange(-1, 2)
+        self.x += self.direction * int(offset_x)
+        self.y += int(offset_y)
+        self.action_x = 0
+        self.action_y = 0
+        self.vx = 0
+        self.vy = 0
+        self.left = int(data.get('disparea.left', 0))
+        self.right = int(data.get('disparea.right', w))
+        self.top = int(data.get('disparea.top', 0))
+        self.bottom = int(data.get('disparea.bottom', h))
         self.script = None
         self.darea = gtk.DrawingArea()
         self.darea.set_events(gtk.gdk.EXPOSURE_MASK|
@@ -226,12 +226,13 @@ class Balloon:
         self.darea.connect('button_release_event', self.button_release)
         self.darea.connect('motion_notify_event', self.motion_notify)
         self.darea.connect('leave_notify_event', self.leave_notify)
-        self.darea.set_size_request(w, h)
         self.darea.show()
         self.window.add(self.darea)
         self.darea.realize()
         self.window.realize()
         self.window.window.set_back_pixmap(None, False)
+        self.set_skin()
+        self.set_position()
         self.layout = None
         if text != 'noscript' and \
            (self.right - self.left) and (self.bottom - self.top):
@@ -242,20 +243,16 @@ class Balloon:
                 fontcolor_b = int(data['font.color'][4:6], 16) / 255.0
                 self.fontcolor = (fontcolor_r, fontcolor_g, fontcolor_b)
             default_font_size = 12 # for Windows environment
-            font_size = int(data.get('font.size', default_font_size))
-            font_size = font_size * 3 / 4 # convert from Windows to GTK+
-            font_size = font_size * self.scale / 100 * pango.SCALE
+            self.font_size = int(data.get('font.size', default_font_size))
             self.layout = pango.Layout(self.darea.get_pango_context())
             self.font_desc = pango.FontDescription()
             self.font_desc.set_family('Sans') # FIXME
-            self.font_desc.set_size(font_size)
             if data.get('font.bold') == 'on':
                 self.font_desc.set_weight(pango.WEIGHT_BOLD)
-            self.layout.set_font_description(self.font_desc)
             self.layout.set_wrap(pango.WRAP_CHAR)
-            self.layout.set_width((self.right - self.left) * 1024)
-        self.slide_vx = int(int(data.get('slide.vx', 0)) * self.scale / 100)
-        self.slide_vy = int(int(data.get('slide.vy', 0)) * self.scale / 100)
+            self.set_layout()
+        self.slide_vx = int(data.get('slide.vx', 0))
+        self.slide_vy = int(data.get('slide.vy', 0))
         self.slide_autostop = int(data.get('slide.autostop', 0))
         if data.get('action.method') in ['sinwave', 'vibrate']:
             action = data['action.method']
@@ -297,12 +294,6 @@ class Balloon:
         self.visible = 0
         self.x_root = None
         self.y_root = None
-        self.action_x = 0
-        self.action_y = 0
-        self.vx = 0
-        self.vy = 0
-        self.text_pos_x = self.left
-        self.text_pos_y = self.top
         self.processed_script = None
         self.processed_text = ''
         self.text = ''
@@ -311,19 +302,68 @@ class Balloon:
         self.script_parser = ninix.script.Parser(error='loose')
         try:
             self.processed_script = self.script_parser.parse(self.script)
-        except ninix.script.ParserError, e:
+        except ninix.script.ParserError as e:
             self.processed_script = None
-            print '-' * 50
-            print e
-            print self.script                                    
+            logging.error('-' * 50)
+            logging.error(e)
+            logging.error(self.script)
         self.timeout_id = glib.timeout_add(10, self.do_idle_tasks)
 
+    def set_position(self):
+        if self.window is None:
+            return
+        new_x = self.base_x + int((self.x + self.action_x + self.vx) * self.scale / 100)
+        new_y = self.base_y + int((self.y + self.action_y + self.vy) * self.scale / 100)
+        self.window.resize_move(new_x, new_y)
+
+    def set_skin(self):
+        if self.window is None:
+            return
+        self.scale = self.get_sakura_status('SurfaceScale') ## FIXME
+        balloon_pixbuf = self.balloon_pixbuf
+        w = balloon_pixbuf.get_width()
+        h = balloon_pixbuf.get_height()
+        w = max(8, int(w * self.scale / 100))
+        h = max(8, int(h * self.scale / 100))
+        balloon_pixbuf = balloon_pixbuf.scale_simple(
+            w, h, gtk.gdk.INTERP_BILINEAR)
+        mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
+        balloon_pixbuf.render_threshold_alpha(
+            mask_pixmap, 0, 0, 0, 0, w, h, 1)
+        self.window.mask = mask_pixmap
+        self.current_balloon_pixbuf = balloon_pixbuf
+        self.darea.set_size_request(w, h)
+        self.base_x, self.base_y = self.get_coordinate(w, h)
+
+    def set_layout(self):
+        if self.window is None:
+            return
+        if self.layout is None:
+            return
+        font_size = self.font_size * 3 / 4 # convert from Windows to GTK+
+        font_size = font_size * self.scale / 100 * pango.SCALE
+        self.font_desc.set_size(font_size)
+        self.layout.set_font_description(self.font_desc)
+        self.layout.set_width(
+            int((self.right - self.left) * 1024 * self.scale / 100))
+
+    def reset_scale(self):
+        if self.window is None:
+            return
+        self.set_skin()
+        self.set_position()
+        self.set_layout()
+        self.darea.queue_draw()
+
+    @property
     def clickerase(self):
         return self.data.get('clickerase', 'on') == 'on'
 
+    @property
     def dragmove_horizontal(self):
         return self.data.get('dragmove.horizontal') == 'on'
 
+    @property
     def dragmove_vertical(self):
         return self.data.get('dragmove.vertical') == 'on'
 
@@ -391,11 +431,11 @@ class Balloon:
         self.quick_session = 0
         try:
             self.processed_script = self.script_parser.parse(self.script)
-        except ninix.script.ParserError, e:
+        except ninix.script.ParserError as e:
             self.processed_script = None
-            print '-' * 50
-            print e
-            print self.script                                    
+            logging.error('-' * 50)
+            logging.error(e)
+            logging.error(self.script)
 
     def get_sakura_is_talking(self):
         talking = 0
@@ -500,27 +540,26 @@ class Balloon:
                                                 % self.action['ref2'])
                                         / self.action['ref2'])
                     if self.action['ref0']:
-                        self.action_y = int(offset * self.scale / 100)
+                        self.action_y = int(offset)
                     else:
-                        self.action_x = int(offset * self.scale / 100)
+                        self.action_x = int(offset)
                 elif self.action['method'] == 'vibrate':
                     offset = (int((time.time() - self.start_time) * 1000) / \
                               self.action['ref2']) % 2
-                    self.action_x = int(offset * self.action['ref0'] * self.scale / 100)
-                    self.action_y = int(offset * self.action['ref1'] * self.scale / 100)
+                    self.action_x = int(offset * self.action['ref0'])
+                    self.action_y = int(offset * self.action['ref1'])
             if (self.slide_vx != 0 or self.slide_vy != 0) and \
                    self.slide_autostop > 0 and \
                    self.slide_autostop * 0.001 + 0.05 <= time.time() - self.start_time:
-                self.vx = self.direction * int((self.slide_autostop / 50.0 + 1) * self.slide_vx * self.scale / 100)
+                self.vx = self.direction * int((self.slide_autostop / 50.0 + 1) * self.slide_vx)
                 self.slide_vx = 0
-                self.vy = int((self.slide_autostop / 50.0 + 1) * self.slide_vy * self.scale / 100)
+                self.vy = int((self.slide_autostop / 50.0 + 1) * self.slide_vy)
                 self.slide_vy = 0
             if self.slide_vx != 0:
-                self.vx = self.direction * int(((time.time() - self.start_time) * self.slide_vx * self.scale / 100) / 50 * 1000.)
+                self.vx = self.direction * int(((time.time() - self.start_time) * self.slide_vx) / 50 * 1000.)
             if self.slide_vy != 0:
-                self.vy = int(((time.time() - self.start_time) * self.slide_vy * self.scale / 100) / 50 * 1000.)
-            self.window.move(int(self.x + self.action_x + self.vx),
-                             int(self.y + self.action_y + self.vy))
+                self.vy = int(((time.time() - self.start_time) * self.slide_vy) / 50 * 1000.)
+            self.set_position()
             if self.processed_script or self.processed_text:
                 self.interpret_script()
         self.talking = 0 if self.talking and not sakura_talking else sakura_talking
@@ -537,7 +576,8 @@ class Balloon:
         cr.paint()
         if self.layout:
             cr.set_source_rgb(*self.fontcolor)
-            cr.move_to(self.left, self.top)
+            cr.move_to(int(self.left * self.scale / 100),
+                       int(self.top * self.scale / 100))
             cr.show_layout(self.layout)
         del cr
 
@@ -609,8 +649,8 @@ class Balloon:
         if event.type == gtk.gdk._2BUTTON_PRESS:
             x = int(event.x * 100 / self.scale)
             y = int(event.y * 100 / self.scale)
-            self.__sakura.notify_event('OnEBMouseDoubleClick', self.name,
-                                       x, y, self.id)
+            self.__sakura.notify_event(
+                'OnEBMouseDoubleClick', self.name, x, y, self.id)
         return True
 
     def button_release(self, widget, event):
@@ -620,35 +660,35 @@ class Balloon:
         y = int(event.y * 100 / self.scale)
         if event.type == gtk.gdk.BUTTON_RELEASE:
             if event.button == 1:
-                self.__sakura.notify_event('OnEBMouseClick', self.name,
-                                           x, y, self.id, 0)
+                self.__sakura.notify_event(
+                    'OnEBMouseClick', self.name, x, y, self.id, 0)
             elif event.button == 3:
-                self.__sakura.notify_event('OnEBMouseClick', self.name,
-                                           x, y, self.id, 1)
-        if self.clickerase():
+                self.__sakura.notify_event(
+                    'OnEBMouseClick', self.name, x, y, self.id, 1)
+        if self.clickerase:
             self.destroy()
         return True
 
     def motion_notify(self, widget, event):
+        scale = self.scale
         if self.x_root is not None and \
            self.y_root is not None:
             x_delta = int(event.x_root - self.x_root)
             y_delta = int(event.y_root - self.y_root)
             if event.state & gtk.gdk.BUTTON1_MASK:
-                if self.dragmove_horizontal():
-                    self.x += x_delta
-                if self.dragmove_vertical():
-                    self.y += y_delta
-                self.window.move(int(self.x + self.action_x + self.vx),
-                                 int(self.y + self.action_y + self.vy))
+                if self.dragmove_horizontal:
+                    self.x += x_delta * 100 / scale
+                if self.dragmove_vertical:
+                    self.y += y_delta * 100 / scale
+                self.set_position()
                 self.x_root = event.x_root
                 self.y_root = event.y_root
         if self.move_notify_time is None or \
            time.time() - self.move_notify_time > 500 * 0.001:
-            x = int(event.x * 100 / self.scale)
-            y = int(event.y * 100 / self.scale)
-            self.__sakura.notify_event('OnEBMouseMove', self.name,
-                                       x, y, self.id)
+            x = int(event.x * 100 / scale)
+            y = int(event.y * 100 / scale)
+            self.__sakura.notify_event(
+                'OnEBMouseMove', self.name, x, y, self.id)
             self.move_notify_time = time.time()
         return True
 
