@@ -20,20 +20,12 @@ import sys
 import os
 import math
 import time
+import logging
 
-if 'DISPLAY' in os.environ:
-    if 'gtk' not in sys.modules:
-        try:
-            import pygtk
-            pygtk.require('2.0')
-        except ImportError:
-            pass
-    import gtk
-    import pango
-    import cairo
-    import ninix.pix
-else:
-    gtk = None
+import gtk
+import pango
+import cairo
+import ninix.pix
 
 from ninix.dll import SAORI
 
@@ -47,9 +39,6 @@ class Saori(SAORI):
         self.graphs = {}
         self.data = {}
 
-    def check_import(self):
-        return 0 if not gtk else 1
-
     def setup(self):
         self.dbpath = os.path.join(self.dir, self.__DBNAME)
         self.graphs = {}
@@ -63,36 +52,36 @@ class Saori(SAORI):
     def read_hanayu_txt(self, dir):
         graphs = {}
         try:
-            f = open(os.path.join(dir, 'hanayu.txt'), 'r')
-            data = {}
-            name = ''
-            tmp_name = ''
-            for line in f:
-                line = line.strip()
-                if not line:
-                    continue
-                if line.startswith('//'):
-                    continue
-                if line.startswith('['): # bln.txt like format
-                    graphs[name] = data
-                    data = {}
-                    end = line.find(']')
-                    if end < 0:
-                        end = len(line)
-                    name = line[1:end]
-                elif line == '{': # surfaces.txt like format
-                    graphs[name] = data
-                    data = {}
-                    name = tmp_name ## FIXME
-                elif line == '}': # surfaces.txt like format
-                    graphs[name] = data
-                    data = {}
-                    name = ''
-                elif ',' in line:
-                    key, value = [x.strip() for x in line.split(',', 1)]
-                    data[key] = value
-                elif not name:
-                    tmp_name = line
+            with open(os.path.join(dir, 'hanayu.txt'), 'r') as f:
+                data = {}
+                name = ''
+                tmp_name = ''
+                for line in f:
+                    line = line.strip()
+                    if not line:
+                        continue
+                    if line.startswith('//'):
+                        continue
+                    if line.startswith('['): # bln.txt like format
+                        graphs[name] = data
+                        data = {}
+                        end = line.find(']')
+                        if end < 0:
+                            end = len(line)
+                        name = line[1:end]
+                    elif line == '{': # surfaces.txt like format
+                        graphs[name] = data
+                        data = {}
+                        name = tmp_name ## FIXME
+                    elif line == '}': # surfaces.txt like format
+                        graphs[name] = data
+                        data = {}
+                        name = ''
+                    elif ',' in line:
+                        key, value = [x.strip() for x in line.split(',', 1)]
+                        data[key] = value
+                    elif not name:
+                        tmp_name = line
             if data:
                 graphs[name] = data
             return graphs
@@ -126,26 +115,24 @@ class Saori(SAORI):
             key, year, month, day = self.time_to_key(current_time, index)
             self.seven_days.append([key, year, month, day, 0.0])
         try:
-            f = open(self.dbpath)
+            with open(self.dbpath) as f:
+                ver = None
+                for line in f:
+                    line = line.strip()
+                    if not line:
+                        continue
+                    if ver is None:
+                        if line == '# Format: v1.0':
+                            ver = 1
+                        continue
+                    if ',' not in line:
+                        continue
+                    key, value = line.split(',', 1)
+                    for index in range(7):
+                        if self.seven_days[index][0] == key:
+                            self.seven_days[index][4] = float(value)
         except:
             return
-        else:
-            ver = None
-            for line in f:
-                line = line.strip()
-                if not line:
-                    continue
-                if ver is None:
-                    if line == '# Format: v1.0':
-                        ver = 1
-                    continue
-                if ',' not in line:
-                    continue
-                key, value = line.split(',', 1)
-                for index in range(7):
-                    if self.seven_days[index][0] == key:
-                        self.seven_days[index][4] = float(value)
-            f.close()
 
     def update_db(self):
         current_time = time.time()
@@ -170,16 +157,14 @@ class Saori(SAORI):
     def write_db(self):
         self.update_db()
         try:
-            f = open(self.dbpath, 'w')
+            with open(self.dbpath, 'w') as f:
+                f.write('# Format: v1.0\n')
+                for index in range(7):
+                    f.write('{0}, {1}\n'.format(
+                            self.seven_days[index][0],
+                            self.seven_days[index][4]))
         except IOError:
-            print 'HANAYU: cannot write database (ignored)'
-        else:
-            f.write('# Format: v1.0\n')
-            for index in range(7):
-                f.write('%s, %s\n' % \
-                        (self.seven_days[index][0],
-                         self.seven_days[index][4]))
-            f.close()
+            logging.error('HANAYU: cannot write database (ignored)')
 
     def execute(self, argument):
         if not argument:
@@ -238,7 +223,7 @@ class Saori(SAORI):
         return self.RESPONSE[204]
 
 
-class Graph:
+class Graph(object):
 
     width = 450
     height = 340
