@@ -20,52 +20,52 @@ import re
 import time
 import random
 import sys
-import StringIO
+import logging
 
 import codecs
 
+from ninix.home import get_normalized_path
+
 ###   READER   ###
 charset = 'Shift_JIS' # default
 
-def read_dict(path, debug=0):
+def read_dict(path):
     global charset
-    f = open(path)
-    lineno = 0
-    buf = []
-    for line in f:
-        lineno = lineno + 1
-        position = (lineno, path)
-        if line.startswith('!KAWA0000'):
-            line = decrypt(line[9:])
-        if not line.strip() or \
-           line.startswith('#') or \
-           line.startswith(':crypt') or \
-           line.startswith(':endcrypt'):
-            continue
-        if ':' not in line:
-            if debug & 4:
-                print 'kawari.py: syntax error at line %d in %s:' % position
-                print line.strip()
-            continue
-        entries, phrases = [x.strip() for x in line.split(':', 1)]
-        if not entries:
-            if debug & 4:
-                print 'kawari.py: syntax error at line %d in %s:' % position
-                print line.strip()
-            continue
-        if not phrases:
-            continue
-        if entries == 'locale':
-            try:
-                codecs.lookup(phrases)
-            except:
-                print 'kawari.py: unsupported charset %s' % phrases
+    with open(path) as f:
+        lineno = 0
+        buf = []
+        for line in f:
+            lineno = lineno + 1
+            position = (lineno, path)
+            if line.startswith('!KAWA0000'):
+                line = decrypt(line[9:])
+            if not line.strip() or \
+                    line.startswith('#') or \
+                    line.startswith(':crypt') or \
+                    line.startswith(':endcrypt'):
+                continue
+            if ':' not in line:
+                logging.debug('kawari.py: syntax error at line {0:d} in {1}:'.format(*position))
+                logging.debug(line.strip())
+                continue
+            entries, phrases = [x.strip() for x in line.split(':', 1)]
+            if not entries:
+                logging.debug('kawari.py: syntax error at line {0:d} in {1}:'.format(*position))
+                logging.debug(line.strip())
+                continue
+            if not phrases:
+                continue
+            if entries == 'locale':
+                try:
+                    codecs.lookup(phrases)
+                except:
+                    logging.error('kawari.py: unsupported charset {0}'.format(phrases))
+                else:
+                    charset = phrases
             else:
-                charset = phrases
-        else:
-            entries = unicode(entries, charset, 'ignore')
-            phrases = unicode(phrases, charset, 'ignore')
-        buf.append((entries, phrases, position))
+                entries = unicode(entries, charset, 'ignore')
+                phrases = unicode(phrases, charset, 'ignore')
+            buf.append((entries, phrases, position))
     return buf
 
 def decrypt(data):
@@ -81,16 +81,15 @@ def encrypt(data):
     line = ''.join(('!KAWA0000', base64.encodestring(''.join(buf))))
     return line.replace('\n', '')
 
-def create_dict(buf, debug=0):
+def create_dict(buf):
     rdict = {} # rules
     kdict = {} # keywords
     for entries, phrases, position in buf:
         parsed_entries = parse_entries(entries)
         parsed_phrases = parse_phrases(phrases)
         if parsed_phrases is None:
-            if debug & 4:
-                print 'kawari.py: syntax error at line %d in %s:' % position
-                print phrases.strip()
+            logging.debug('kawari.py: syntax error at line {0:d} in {1}:'.format(*position))
+            logging.debug(phrases.strip())
             continue
         if entries.startswith('['):
             add_phrases(kdict, tuple(parsed_entries), parsed_phrases)
@@ -146,7 +145,7 @@ def parse(data, start, stop_pattern=None):
             break
         elif data[i] == '"':
             i, text = parse_quotes(data, i)
-            buf.append('"%s"' % text)
+            buf.append('"{0}"'.format(text))
         elif data[i:i + 2] == '${':
             i, text = parse_reference(data, i)
             buf.append(text)
@@ -219,7 +218,7 @@ def parse_inline_script(data, start):
         #end
         if data[i] == '"':
             i, text = parse_quotes(data, i)
-            buf.append('"%s"' % text)
+            buf.append('"{0}"'.format(text))
         elif data[i:i + 2] == '${':
             i, text = parse_reference(data, i)
             buf.append(text)
@@ -260,29 +259,28 @@ def parse_text(data, start, stop_pattern=None):
     return i, data[start:i]
 
 def read_local_script(path):
-    f = open(path)
     rdict = {}
     kdict = {}
-    for line in f.readlines():
-        if line.startswith('#'):
-            rdict[unicode(line.strip(), charset, 'ignore')] = \
-                 [unicode(f.readline().strip(), charset, 'ignore')]
+    with open(path) as f:
+        for line in f.readlines():
+            if line.startswith('#'):
+                rdict[unicode(line.strip(), charset, 'ignore')] = \
+                    [unicode(f.readline().strip(), charset, 'ignore')]
     return rdict, kdict
 
 
 ###   KAWARI   ###
 
-class Kawari:
+class Kawari(object):
 
     MAXDEPTH = 30
 
-    def __init__(self, prefix, pathlist, rdictlist, kdictlist, debug=0):
+    def __init__(self, prefix, pathlist, rdictlist, kdictlist):
         self.prefix = prefix
         self.pathlist = pathlist
         self.rdictlist = rdictlist
         self.kdictlist = kdictlist
         self.system_entries = {}
-        self.debug = debug
         self.expr_parser = ExprParser()
         #begin specification bug work-around (2/3)
         self.expr_parser.kawari = self
@@ -314,7 +312,7 @@ class Kawari:
 
     def getstring(self, name):
         name = unicode(name, charset, 'ignore')
-        return self.get('resource.%s' % name).encode(charset, 'ignore')
+        return self.get('resource.{0}'.format(name)).encode(charset, 'ignore')
 
     # SHIORI/2.2 API
     def get_event_response(self, event,
@@ -329,8 +327,8 @@ class Kawari:
         for i in range(8):
             if ref[i] is not None:
                 value = ref[i]
-                self.system_entries['system.Reference%d' % i] = value
-                self.system_entries['system-Reference%d' % i] = value
+                self.system_entries['system.Reference{0:d}'.format(i)] = value
+                self.system_entries['system-Reference{0:d}'.format(i)] = value
         script = None
         if event == 'OnCommunicate':
             self.system_entries['system.Sender'] = ref[0]
@@ -429,8 +427,7 @@ class Kawari:
 
     # internal
     def clear(self, name):
-        if self.debug & 128:
-            print '*** clear("%s")' % name.encode('utf-8', 'ignore')
+        logging.debug('*** clear("{0}")'.format(name.encode('utf-8', 'ignore')))
         for dic in self.rdictlist:
             if name in dic:
                 del dic[name]
@@ -445,9 +442,9 @@ class Kawari:
         return dic
 
     def unshift(self, name, value):
-        if self.debug & 128:
-            print ('*** unshift("%s", "%s")' % (name, value)).encode('utf-8',
-                                                                     'ignore')
+        logging.debug(
+            ('*** unshift("{0}", "{1}")'.format(name, value)).encode('utf-8',
+                                                                     'ignore'))
         dic = self.get_internal_dict(name)
         i, segments = parse(value, 0)
         dic[name].insert(0, segments)
@@ -455,15 +452,15 @@ class Kawari:
     def shift(self, name):
         dic = self.get_internal_dict(name)
         value = self.expand(dic[name].pop(0))
-        if self.debug & 128:
-            print ('*** shift("%s") => "%s"' % (name, value)).encode('utf-8',
-                                                                     'ignore')
+        logging.debug(
+            ('*** shift("{0}") => "{1}"'.format(name, value)).encode('utf-8',
+                                                                     'ignore'))
         return value
 
     def push(self, name, value):
-        if self.debug & 128:
-            print ('*** push("%s", "%s")' % (name, value)).encode('utf-8',
-                                                                  'ignore')
+        logging.debug(
+            ('*** push("{0}", "{1}")'.format(name, value)).encode('utf-8',
+                                                                  'ignore'))
         dic = self.get_internal_dict(name)
         i, segments = parse(value, 0)
         dic[name].append(segments)
@@ -471,9 +468,9 @@ class Kawari:
     def pop(self, name):
         dic = self.get_internal_dict(name)
         value = self.expand(dic[name].pop())
-        if self.debug & 128:
-            print ('*** pop("%s") => "%s"' % (name, value)).encode('utf-8',
-                                                                   'ignore')
+        logging.debug(
+            ('*** pop("{0}") => "{1}"'.format(name, value)).encode('utf-8',
+                                                                   'ignore'))
         return value
 
     def set(self, name, value):
@@ -491,13 +488,13 @@ class Kawari:
             else:
                 selection = self.select_compound_phrase(name)
             if selection is None:
-                if self.debug & 8:
-                    print '${%s} not found' % name.encode('utf-8', 'ignore')
+                logging.debug('${{{0}}} not found'.format(name.encode('utf-8',
+                                                                      'ignore')))
                 return default
             segments, context = selection
-        if self.debug & 16:
-            print name.encode('utf-8', 'ignore'), '=>', \
-                  ''.join(segments).encode('utf-8', 'ignore')
+        logging.debug(
+            ''.join((name.encode('utf-8', 'ignore'), '=>',
+                     ''.join(segments).encode('utf-8', 'ignore'))))
         return self.expand(segments, context, depth)
 
     def parse_all(self, data, start=0):
@@ -617,8 +614,7 @@ class Kawari:
             if i + 1 < len(segments) and segments[i + 1] == '$(endif)':
                 i += 1
             else:
-                if self.debug & 32:
-                    print 'kawari.py: syntax error: $(endif) expected'
+                logging.debug('kawari.py: syntax error: $(endif) expected')
                 return i, '' # syntax error
             return i, self.exec_old_if(if_block, then_block, else_block)
         # execute command(s)
@@ -629,23 +625,25 @@ class Kawari:
             if argv[0] == 'silent':
                 if len(argv) == 1:
                     values = []
-                elif self.debug & 32:
-                    print 'kawari.py: syntax error:', \
-                          segments[i].encode('utf-8', 'ignore')
+                else:
+                    logging.debug(
+                        ''.join(('kawari.py: syntax error:',
+                                 segments[i].encode('utf-8', 'ignore'))))
                 continue
             handler = self.kis_commands.get(argv[0])
             try:
                 if handler is None:
                     raise RuntimeError, 'invalid command'
                 values.append(handler(self, argv))
-            except RuntimeError, message:
-                if self.debug & 32:
-                    print 'kawari.py: %s: %s' % \
-                          (message, segments[i].encode('utf-8', 'ignore'))
+            except RuntimeError as message:
+                logging.debug(
+                    'kawari.py: {0}: {1}'.format(
+                        message, segments[i].encode('utf-8', 'ignore')))
         result = ''.join(values)
-        if self.debug & 64:
-            print '>>>', segments[i].encode('utf-8', 'ignore')
-            print ''.join(('"', result.encode('utf-8', 'ignore'), '"'))
+        logging.debug(
+            ''.join(('>>>', segments[i].encode('utf-8', 'ignore'))))
+        logging.debug(
+            ''.join(('"', result.encode('utf-8', 'ignore'), '"')))
         return i, result
 
     def split_commands(self, data):
@@ -697,17 +695,19 @@ class Kawari:
         if else_block:
             i, else_block = parse(else_block, 0)
         result = self.exec_if(if_block, then_block, else_block)
-        if self.debug & 64:
-            if else_block:
-                print '>>> $(if %s)$(then %s)$(else %s)$(endif)' % (
+        if else_block:
+            logging.debug(
+                '>>> $(if {0})$(then {1})$(else {2})$(endif)'.format(
                     ''.join(if_block).encode('utf-8', 'ignore'),
                     ''.join(then_block).encode('utf-8', 'ignore'),
-                    ''.join(else_block).encode('utf-8', 'ignore'))
-            else:
-                print '>>> $(if %s)$(then %s)$(endif)' % (
+                    ''.join(else_block).encode('utf-8', 'ignore')))
+        else:
+            logging.debug(
+                '>>> $(if {0})$(then {1})$(endif)'.format(
                     ''.join(if_block).encode('utf-8', 'ignore'),
-                    ''.join(then_block).encode('utf-8', 'ignore'))
-            print ''.join(('"', result.encode('utf-8', 'ignore'), '"'))
+                    ''.join(then_block).encode('utf-8', 'ignore')))
+        logging.debug(
+            ''.join(('"', result.encode('utf-8', 'ignore'), '"')))
         return result
 
     def exec_if(self, if_block, then_block, else_block):
@@ -863,13 +863,13 @@ class Kawari:
         n = 0
         for word in word_list:
             n += 1
-            entry = '%s.%d' % (name, n)
+            entry = '{0}.{1:d}'.format(name, n)
             self.set(entry, word)
         self.set(''.join((name, '.size')), str(n))
         return ''
 
     def get_dict_path(self, path):
-        path = path.replace('\\', '/').lower()
+        path = get_normalized_path(path, encode=0)
         if not path:
             raise RuntimeError, 'invalid argument'
         if path.startswith('/'):
@@ -881,7 +881,7 @@ class Kawari:
             raise RuntimeError, 'syntax error'
         path = self.get_dict_path(self.expand(argv[1]))
         try:
-            rdict, kdict = create_dict(read_dict(path, self.debug))
+            rdict, kdict = create_dict(read_dict(path))
         except IOError:
             raise RuntimeError, 'cannot read file'
         if path in self.pathlist:
@@ -899,22 +899,21 @@ class Kawari:
             raise RuntimeError, 'syntax error'
         path = self.get_dict_path(self.expand(argv[1]))
         try:
-            f = open(path, 'w')
-            f.write('#\r\n# Kawari save file\r\n#\r\n')
-            for i in range(2, len(argv)):
-                name = self.expand(argv[i])
-                if not name.strip():
-                    continue
-                buf = []
-                for segments in self.enumerate(name):
-                    buf.append(''.join(segments))
-                name = name.encode(charset, 'ignore')
-                line = ''.join((name, ' : ',
-                                ' , '.join(buf).encode(charset, 'ignore')))
-                if crypt:
-                    line = encrypt(line)
-                f.write('# Entry %s\r\n%s\r\n' % (name, line))
-            f.close()
+            with open(path, 'w') as f:
+                f.write('#\r\n# Kawari save file\r\n#\r\n')
+                for i in range(2, len(argv)):
+                    name = self.expand(argv[i])
+                    if not name.strip():
+                        continue
+                    buf = []
+                    for segments in self.enumerate(name):
+                        buf.append(''.join(segments))
+                    name = name.encode(charset, 'ignore')
+                    line = ''.join((name, ' : ',
+                                    ' , '.join(buf).encode(charset, 'ignore')))
+                    if crypt:
+                        line = encrypt(line)
+                    f.write('# Entry {0}\r\n{1}\r\n'.format(name, line))
         except IOError:
             raise RuntimeError, 'cannot write file'
         return ''
@@ -927,14 +926,15 @@ class Kawari:
             raise RuntimeError, 'syntax error'
         path = self.get_dict_path(self.expand(argv[1]))
         try:
-            linelist = open(path).readlines()
+            with open(path) as f:
+                linelist = f.readlines()
         except IOError:
             raise RuntimeError, 'cannot read file'
         name = self.expand(argv[2])
         n = 0
         for line in linelist:
             n += 1
-            entry = '%s.%d' % (name, n)
+            entry = '{0}.{1:d}'.format(name, n)
             if line.endswith('\r\n'):
                 line = line[:-2]
             elif line.endswith('\r') or line.endswith('\n'):
@@ -1004,47 +1004,47 @@ class Kawari:
 
     def exec_date(self, argv):
         if len(argv) == 1:
-            format = '%y/%m/%d %H:%M:%S'
+            format_ = '%y/%m/%d %H:%M:%S'
         else:
-            format = ' '.join([self.expand(s) for s in argv[1:]])
+            format_ = ' '.join([self.expand(s) for s in argv[1:]])
         buf = []
         i = 0
-        j = len(format)
+        j = len(format_)
         now = time.localtime(time.time())
         while i < j:
-            if format[i] == '%':
+            if format_[i] == '%':
                 i += 1
                 if i < j:
-                    c = format[i]
+                    c = format_[i]
                     i += 1
                 else:
                     break
                 if c in ['y', 'Y']: # year (4 columns)
-                    buf.append('%04d' % now[0])
+                    buf.append('{0:04d}'.format(now[0]))
                 elif c == 'm': # month (01 - 12)
-                    buf.append('%02d' % now[1])
+                    buf.append('{0:02d}'.format(now[1]))
                 elif c == 'n': # month (1 - 12)
                     buf.append(str(now[1]))
                 elif c == 'd': # day (01 - 31)
-                    buf.append('%02d' % now[2])
+                    buf.append('{0:02d}'.format(now[2]))
                 elif c == 'e': # day (1 - 31)
                     buf.append(str(now[2]))
                 elif c == 'H': # hour (00 - 23)
-                    buf.append('%02d' % now[3])
+                    buf.append('{0:02d}'.format(now[3]))
                 elif c == 'k': # hour (0 - 23)
                     buf.append(str(now[3]))
                 elif c == 'M': # minute (00 - 59)
-                    buf.append('%02d' % now[4])
+                    buf.append('{0:02d}'.format(now[4]))
                 elif c == 'N': # minute (0 - 59)
                     buf.append(str(now[4]))
                 elif c == 'S': # second (00 - 59)
-                    buf.append('%02d' % now[5])
+                    buf.append('{0:02d}'.format(now[5]))
                 elif c == 'r': # second (0 - 59)
                     buf.append(str(now[5]))
                 elif c == 'w': # weekday (0 = Sunday)
                     buf.append(str([1, 2, 3, 4, 5, 6, 0][now[6]]))
                 elif c == 'j': # Julian day (001 - 366)
-                    buf.append('%03d' % now[7])
+                    buf.append('{0:03d}'.format(now[7]))
                 elif c == 'J': # Julian day (1 - 366)
                     buf.append(str(now[7]))
                 elif c == '%':
@@ -1053,7 +1053,7 @@ class Kawari:
                     buf.append('%')
                     i -= 1
             else:
-                buf.append(format[i])
+                buf.append(format_[i])
                 i += 1
         return ''.join(buf)
 
@@ -1317,23 +1317,23 @@ class Kawari:
         'callsaorix':  None,
         }
 
+
 ###   EXPR PARSER   ###
 
 class ExprError(ValueError):
     pass
 
-class ExprParser:
+
+class ExprParser(object):
 
     def __init__(self):
-        self.debug = 0
-        # bit 0 = trace get_expr()
-        # bit 1 = trace all "get_" functions
+        pass
 
     def show_progress(self, func, buf):
         if buf is None:
-            print '%s() -> syntax error' % func
+            logging.debug('{0}() -> syntax error'.format(func))
         else:
-            print '%s() -> %s' % (func, buf)
+            logging.debug('{0}() -> {1}'.format(func, buf))
 
     re_token = re.compile('[():|&*/%+-]|[<>]=?|[!=]?=|match|index|findpos|find|substr|length|quote|(\\s+)')
 
@@ -1401,8 +1401,7 @@ class ExprParser:
         buf = self.get_or_expr()
         if not self.done():
             raise ExprError
-        if self.debug & 1:
-            self.show_progress('get_expr', buf)
+        self.show_progress('get_expr', buf)
         return buf
 
     def get_or_expr(self):
@@ -1418,8 +1417,7 @@ class ExprParser:
                 break
         if len(buf) == 2:
             buf = buf[1]
-        if self.debug & 2:
-            self.show_progress('get_or_expr', buf)
+        self.show_progress('get_or_expr', buf)
         return buf
 
     def get_and_expr(self):
@@ -1435,8 +1433,7 @@ class ExprParser:
                 break
         if len(buf) == 2:
             buf = buf[1]
-        if self.debug & 2:
-            self.show_progress('get_and_expr', buf)
+        self.show_progress('get_and_expr', buf)
         return buf
 
     def get_cmp_expr(self):
@@ -1451,8 +1448,7 @@ class ExprParser:
             buf.append(self.get_add_expr())
         if len(buf) == 2:
             buf = buf[1]
-        if self.debug & 2:
-            self.show_progress('get_cmp_expr', buf)
+        self.show_progress('get_cmp_expr', buf)
         return buf
 
     def get_add_expr(self):
@@ -1468,8 +1464,7 @@ class ExprParser:
                 break
         if len(buf) == 2:
             buf = buf[1]
-        if self.debug & 2:
-            self.show_progress('get_add_expr', buf)
+        self.show_progress('get_add_expr', buf)
         return buf
 
     def get_mul_expr(self):
@@ -1485,8 +1480,7 @@ class ExprParser:
                 break
         if len(buf) == 2:
             buf = buf[1]
-        if self.debug & 2:
-            self.show_progress('get_mul_expr', buf)
+        self.show_progress('get_mul_expr', buf)
         return buf
 
     def get_mat_expr(self):
@@ -1501,8 +1495,7 @@ class ExprParser:
             buf.append(self.get_str_expr())
         if len(buf) == 2:
             buf = buf[1]
-        if self.debug & 2:
-            self.show_progress('get_mat_expr', buf)
+        self.show_progress('get_mat_expr', buf)
         return buf
 
     def get_str_expr(self):
@@ -1528,8 +1521,7 @@ class ExprParser:
                 buf.extend(self.get_str_seq())
         else:
             buf = self.get_sub_expr()
-        if self.debug & 2:
-            self.show_progress('get_str_expr', buf)
+        self.show_progress('get_str_expr', buf)
         return buf
 
     def get_sub_expr(self):
@@ -1547,8 +1539,7 @@ class ExprParser:
         else:
             buf = [self.LITERAL]
             buf.extend(self.get_str_seq())
-        if self.debug & 2:
-            self.show_progress('get_sub_expr', buf)
+        self.show_progress('get_sub_expr', buf)
         return buf
 
     def get_str_seq(self):
@@ -1588,12 +1579,12 @@ class ExprParser:
 
 ###   API   ###
 
-DICT_FILE, INI_FILE = range(2)
+DICT_FILE, INI_FILE = list(range(2))
 
-def list_dict(kawari_dir, saori_ini={}, debug=0):
-    return scan_ini(kawari_dir, 'kawari.ini', saori_ini, debug)
+def list_dict(kawari_dir, saori_ini={}):
+    return scan_ini(kawari_dir, 'kawari.ini', saori_ini)
 
-def scan_ini(kawari_dir, filename, saori_ini, debug=0):
+def scan_ini(kawari_dir, filename, saori_ini):
     buf = []
     ini_path = os.path.join(kawari_dir, filename)
     try:
@@ -1603,18 +1594,19 @@ def scan_ini(kawari_dir, filename, saori_ini, debug=0):
     read_as_dict = 0
     for entry, value, position in line_list:
         if entry == 'dict':
-            filename = value.replace('\\', '/').lower().encode('utf-8')
+            filename = get_normalized_path(value)
             path = os.path.join(kawari_dir, filename)
             try:
-                open(path).read(64)
-            except IOError, (errno, message):
-                if debug & 4:
-                    print 'kawari.py: read error:', path
+                with open(path) as f:
+                    f.read(64)
+            except IOError as e:
+                errno, message = e.args
+                logging.debug('kawari.py: read error: {0}'.format(path))
                 continue
             buf.append((DICT_FILE, path))
         elif entry == 'include':
-            filename = value.replace('\\', '/').lower().encode('utf-8')
-            buf.extend(scan_ini(kawari_dir, filename, saori_ini, debug))
+            filename = get_normalized_path(value)
+            buf.extend(scan_ini(kawari_dir, filename, saori_ini))
         elif entry == 'set':
             read_as_dict = 1
         elif entry in ['randomseed', 'debug', 'security', 'set']:
@@ -1628,8 +1620,8 @@ def scan_ini(kawari_dir, filename, saori_ini, debug=0):
             else:
                 option = 'loadoncall'
             saori_ini[alias] = [path, option]
-        elif debug & 4:
-            print 'kawari.py: unknown entry:', entry
+        else:
+            logging.debug('kawari.py: unknown entry: {0}'.format(entry))
     if read_as_dict:
         buf.append((INI_FILE, ini_path))
     return buf
@@ -1652,8 +1644,7 @@ def read_ini(path):
 
 class Shiori(Kawari):
 
-    def __init__(self, dll_name, debug=0):
-        self.debug = debug
+    def __init__(self, dll_name):
         self.dll_name = dll_name
         self.saori_list = {}
         self.saori_ini = {}
@@ -1671,20 +1662,18 @@ class Shiori(Kawari):
         rdictlist = [{}]
         kdictlist = [{}]
         self.saori_ini = {}
-        for file_type, path in list_dict(kawari_dir, self.saori_ini, self.debug):
+        for file_type, path in list_dict(kawari_dir, self.saori_ini):
             pathlist.append(path)
             if file_type == INI_FILE:
-                rdict, kdict = create_dict(read_ini(path), self.debug)
+                rdict, kdict = create_dict(read_ini(path))
             elif is_local_script(path):
                 rdict, kdict = read_local_script(path)
             else:
-                rdict, kdict = create_dict(read_dict(path, self.debug),
-                                           self.debug)
+                rdict, kdict = create_dict(read_dict(path))
             rdictlist.append(rdict)
             kdictlist.append(kdict)
-        Kawari.__init__(self, kawari_dir, pathlist, rdictlist, kdictlist,
-                        self.debug)
-        for value in self.saori_ini.itervalues():
+        Kawari.__init__(self, kawari_dir, pathlist, rdictlist, kdictlist)
+        for value in self.saori_ini.values():
             if value[1] == 'preload':
                 head, tail = os.path.split(value[0].replace('\\', '/'))
                 self.saori_load(value[0], os.path.join(self.kawari_dir, head))
@@ -1707,17 +1696,17 @@ class Shiori(Kawari):
         return result
 
     def show_description(self):
-        sys.stdout.write(
+        logging.info(
             'Shiori: KAWARI compatible module for ninix\n'
             '        Copyright (C) 2001, 2002 by Tamito KAJIYAMA\n'
             '        Copyright (C) 2002, 2003 by MATSUMURA Namihiko\n'
             '        Copyright (C) 2002-2011 by Shyouzou Sugitani\n'
-            '        Copyright (C) 2003 by Shun-ichi TAHARA\n')
+            '        Copyright (C) 2003 by Shun-ichi TAHARA')
 
     def request(self, req_string):
-        header = StringIO.StringIO(req_string)
+        header = req_string.splitlines()
         req_header = {}
-        line = header.readline()
+        line = header.pop(0)
         if line:
             line = line.strip()
             req_list = line.split()
@@ -1739,9 +1728,7 @@ class Shiori(Kawari):
         result = ''
         to = None
         if 'ID' in req_header:
-            if req_header['ID'] == 'charset':
-                result = charset
-            elif req_header['ID'] == 'dms':
+            if req_header['ID'] == 'dms':
                 result = self.getdms()
             elif req_header['ID'] == 'OnAITalk':
                 result = self.getaistringrandom()
@@ -1779,9 +1766,10 @@ class Shiori(Kawari):
             to = self.communicate_to()
         result = 'SHIORI/3.0 200 OK\r\n' \
                  'Sender: Kawari\r\n' \
-                 'Value: %s\r\n' % result
+                 'Charset: {0}\r\n' \
+                 'Value: {1}\r\n'.format(charset, result)
         if to is not None:
-            result = ''.join((result, 'Reference0: %s\r\n' % to))
+            result = ''.join((result, 'Reference0: {0}\r\n'.format(to)))
         result = ''.join((result, '\r\n'))
         return result            
 
@@ -1825,15 +1813,15 @@ class Shiori(Kawari):
         req = 'EXECUTE SAORI/1.0\r\n' \
               'Sender: KAWARI\r\n' \
               'SecurityLevel: local\r\n' \
-              'Charset: %s\r\n' % charset
+              'Charset: {0}\r\n'.format(charset)
         for i in range(2, len(argv)):
             req = ''.join((req,
-                           'Argument%s: %s\r\n' % (i - 2, self.expand(argv[i]))))
+                           'Argument{0}: {1}\r\n'.format(i - 2, self.expand(argv[i]))))
         req = ''.join((req, '\r\n'))
         response = self.saori_request(self.saori_ini[alias][0],
                                       req.encode(charset, 'ignore'))
-        header = StringIO.StringIO(response)
-        line = header.readline()
+        header = response.splitlines()
+        line = header.pop(0)
         if line:
             line = line.strip()
             if ' ' in line:
@@ -1878,12 +1866,12 @@ class Shiori(Kawari):
               'SecurityLevel: local\r\n'
         for i in range(3, len(argv)):
             req = ''.join((req,
-                           'Argument%s: %s\r\n' % (i - 3, self.expand(argv[i]))))
+                           'Argument{0}: {1}\r\n'.format(i - 3, self.expand(argv[i]))))
         req = ''.join((req, '\r\n'))
         response = self.saori_request(self.saori_ini[alias][0],
                                       req.encode(charset, 'ignore'))
-        header = StringIO.StringIO(response)
-        line = header.readline()
+        header = response.splitlines()
+        line = header.pop(0)
         if line:
             line = line.strip()
             if ' ' in line:
@@ -1899,10 +1887,10 @@ class Shiori(Kawari):
                     saori_header.append(key)
                     saori_value[key] = value
         result = {}
-        for key, value in saori_value.iteritems():
+        for key, value in saori_value.items():
             if key.startswith('Value'):
                 result[key] = value
-        for key, value in result.iteritems():
+        for key, value in result.items():
             self.set(''.join((entry, '.', key)), value)
         if self.saori_ini[alias][1] == 'noresident':
             self.saori_unload(self.saori_ini[alias][0])
@@ -1932,51 +1920,45 @@ class Shiori(Kawari):
         return result
 
 
-def kawari_open(kawari_dir, debug=0):
+def kawari_open(kawari_dir):
     pathlist = [None]
     rdictlist = [{}]
     kdictlist = [{}]
-    for file_type, path in list_dict(kawari_dir, debug=debug):
+    for file_type, path in list_dict(kawari_dir):
         pathlist.append(path)
         if file_type == INI_FILE:
-            rdict, kdict = create_dict(read_ini(path), debug)
+            rdict, kdict = create_dict(read_ini(path))
         elif is_local_script(path):
             rdict, kdict = read_local_script(path)
         else:
-            rdict, kdict = create_dict(read_dict(path, debug), debug)
+            rdict, kdict = create_dict(read_dict(path))
         rdictlist.append(rdict)
         kdictlist.append(kdict)
-    return Kawari(kawari_dir, pathlist, rdictlist, kdictlist, debug)
+    return Kawari(kawari_dir, pathlist, rdictlist, kdictlist)
 
 def is_local_script(path):
-    line = open(path).readline()
+    with open(path) as f:
+        line = f.readline()
     return line.startswith('[SAKURA]')
 
 ###   TEST   ###
 
 def test():
-    import getopt
-    try:
-        options, argv = getopt.getopt(sys.argv[1:], 'd:', ['debug='])
-    except getopt.error, e:
-        raise SystemExit, ''.join((str(e), '\n'))
-    if argv:
-        kawari_dir = argv[0]
+    logger = logging.getLogger()
+    logger.setLevel(logging.DEBUG) # XXX
+    if len(sys.argv) >= 2:
+        kawari_dir = sys.argv[1]
     else:
         kawari_dir = os.curdir
-    debug = 4 + 8 + 16 + 32 + 64 + 128
-    for opt, val in options:
-        if opt in ['-d', '--debug']:
-            debug = int(val)
     print 'reading kawari.ini...'
-    kawari = kawari_open(kawari_dir, debug)
+    kawari = kawari_open(kawari_dir)
     for rdict in kawari.rdictlist:
-        for k, v in rdict.iteritems():
+        for k, v in rdict.items():
             print k.encode('utf-8', 'ignore')
             for p in v:
                 print ''.join(('\t', ''.join(p).encode('utf-8', 'ignore')))
     for kdict in kawari.kdictlist:
-        for k, v in kdict.iteritems():
+        for k, v in kdict.items():
             print ''.join(
                 ('[ "', '", "'.join(k).encode('utf-8', 'ignore'), '" ]'))
             for p in v:
