@@ -15,6 +15,7 @@
 
 import os
 import sys
+import logging
 
 try:
     import _kawari8
@@ -22,7 +23,7 @@ except:
     _kawari8 = None
 
 
-class Shiori:
+class Shiori(object):
 
     def __init__(self, dll_name):
         self.dll_name = dll_name
@@ -38,11 +39,11 @@ class Shiori:
         return result
 
     def show_description(self):
-        sys.stdout.write(
+        logging.info(
             'Shiori: Real Kawari8 loader for ninix\n'
             '        Copyright (C) 2002, 2003 by ABE Hideaki\n'
             '        Copyright (C) 2002-2011 by Shyouzou Sugitani\n'
-            '        Copyright (C) 2002, 2003 by MATSUMURA Namihiko\n')
+            '        Copyright (C) 2002, 2003 by MATSUMURA Namihiko')
 
     def load(self, topdir):
         self.dir = topdir
@@ -59,10 +60,7 @@ class Shiori:
                                  self.saori_request)
             result = _kawari8.load(topdir)
             self.handle = result
-            if result != 0:
-                return 1
-            else:
-                return 0
+            return 1 if result != 0 else 0
         else:
             return 0
 
