@@ -13,11 +13,17 @@
 #
 
 import os
+import urllib
+import logging
 
-import pygst
-pygst.require("0.10")
-import gst
+try:
+    ##import pygst
+    ##pygst.require('0.10') # This breaks sys.path. :-(
+    import gst
+except:
+    gst = None
 
+from ninix.home import get_normalized_path
 from ninix.dll import SAORI
 
 
@@ -25,12 +31,12 @@ class Saori(SAORI):
 
     def __init__(self):
         SAORI.__init__(self)
-        self.player = gst.element_factory_make("playbin", "player")
-        fakesink = gst.element_factory_make("fakesink", "fakesink")
-        self.player.set_property("video-sink", fakesink)
+        self.player = gst.element_factory_make('playbin', 'player')
+        fakesink = gst.element_factory_make('fakesink', 'fakesink')
+        self.player.set_property('video-sink', fakesink)
         bus = self.player.get_bus()
         bus.add_signal_watch()
-        bus.connect("message", self.on_message)
+        bus.connect('message', self.on_message)
         self.filepath = None
         self.__sakura = None
         self.state = 0 # 0/1/2 - stop/playing/paused
@@ -39,10 +45,7 @@ class Saori(SAORI):
         self.__sakura = sakura
 
     def check_import(self):
-        if self.__sakura is not None:
-            return 1
-        else:
-            return 0
+        return 1 if self.__sakura is not None and gst is not None else 0
 
     def finalize(self):
         self.player.set_state(gst.STATE_NULL)
@@ -66,12 +69,13 @@ class Saori(SAORI):
                     self.player.set_state(gst.STATE_PAUSED)
                     return self.RESPONSE[204]
                 if self.filepath is not None and os.path.isfile(self.filepath):
-                    self.player.set_property("uri", "file://" + self.filepath)
+                    self.player.set_property(
+                        'uri', 'file://' + urllib.quote(self.filepath))
                     self.player.set_state(gst.STATE_PLAYING)
         elif argc == 2:
             if argv[0] == 'load':
                 self.player.set_state(gst.STATE_NULL)
-                filename = argv[1].replace('\\', '/').lower()
+                filename = get_normalized_path(argv[1])
                 if os.path.isabs(filename):
                     return self.RESPONSE[400]
                 self.filepath = os.path.join(self.__sakura.get_prefix(),
@@ -87,5 +91,5 @@ class Saori(SAORI):
         elif t == gst.MESSAGE_ERROR:
             self.player.set_state(gst.STATE_NULL)
             err, debug = message.parse_error()
-            print "Error: %s" % err, debug
+            logging.error('Error: {0}, {1}'.format(err, debug))
             self.state = 0
