@@ -22,6 +22,7 @@ import os
 import re
 import StringIO
 import sys
+import logging
 import time
 import random
 
@@ -30,6 +31,8 @@ try:
 except:
     chardet = None
 
+from ninix.home import get_normalized_path
+
 
 class MisakaError(ValueError):
     pass
@@ -39,79 +42,79 @@ def lexical_error(path=None, position=None):
     if path is None and position is None:
         error = 'lexical error'
     elif path is None:
-        at = 'line %d, column %d' % (position)
-        error = 'lexical error at %s' % at
+        at = 'line {0:d}, column {1:d}'.format(*position)
+        error = 'lexical error at {0}'.format(at)
     elif position is None:
-        error = 'lexical error in %s' % path
+        error = 'lexical error in {0}'.format(path)
     else:
-        at = 'line %d, column %d' % (position)
-        error = 'lexical error at %s in %s' % (at, path)
+        at = 'line {0:d}, column {1:d}'.format(*position)
+        error = 'lexical error at {0} in {1}'.format(at, path)
     raise MisakaError, error
 
 def syntax_error(message, path=None, position=None):
     if path is None and position is None:
-        error = 'syntax error (%s)' % message
+        error = 'syntax error ({0})'.format(message)
     elif path is None:
-        at = 'line %d, column %d' % position
-        error = 'syntax error at %s (%s)' % (at, message)
+        at = 'line {0:d}, column {1:d}'.format(*position)
+        error = 'syntax error at {0} ({1})'.format(at, message)
     elif position is None:
-        error = 'syntax error in %s (%s)' % (path, message)
+        error = 'syntax error in {0} ({1})'.format(path, message)
     else:
-        at = 'line %d, column %d' % position
-        error = 'syntax error at %s in %s (%s)' % (at, path, message)
+        at = 'line {0:d}, column {1:d}'.format(*position)
+        error = 'syntax error at {0} in {1} ({2})'.format(at, path, message)
     raise MisakaError, error
 
 def list_dict(top_dir):
     buf = []
     try:
         filelist, debug, error = read_misaka_ini(top_dir)
-    except Exception, e:
+    except Exception as e:
         filelist = []
     for filename in filelist:
         buf.append(os.path.join(top_dir, filename))
     return buf
 
-def read_misaka_ini(top_dir, _debug=0):
+def read_misaka_ini(top_dir):
     path = os.path.join(top_dir, 'misaka.ini')
-    f = open(path)
-    filelist = []
-    debug = 0
-    error = 0
-    while 1:
-        line = f.readline()
-        if not line:
-            break
-        line = line.strip()
-        if not line or line.startswith('//'):
-            continue
-        if line == 'dictionaries':
+    with open(path) as f:
+        filelist = []
+        debug = 0
+        error = 0
+        while 1:
             line = f.readline()
-            if line.strip() != '{':
-                syntax_error('expected an open brace', path)
-            while 1:
+            if not line:
+                break
+            line = line.strip()
+            if not line or line.startswith('//'):
+                continue
+            if line == 'dictionaries':
                 line = f.readline()
-                if not line:
-                    syntax_error('unexpected end of file', path)
-                line = line.strip()
-                if line == '}':
-                    break
-                if not line or line.startswith('//'):
-                    continue
-                filelist.append(line.replace('\\', '/').lower())
-        elif line == 'debug,0':
-            debug = 0
-        elif line == 'debug,1':
-            debug = 1
-        elif line == 'error,0':
-            error = 0
-        elif line == 'error,1':
-            error = 1
-        elif line == 'propertyhandler,0':
-            pass
-        elif line == 'propertyhandler,1':
-            pass
-        elif _debug & 4:
-            print "misaka.py: unknown directive '%s'" % line
+                if line.strip() != '{':
+                    syntax_error('expected an open brace', path)
+                while 1:
+                    line = f.readline()
+                    if not line:
+                        syntax_error('unexpected end of file', path)
+                    line = line.strip()
+                    if line == '}':
+                        break
+                    if not line or line.startswith('//'):
+                        continue
+                    filelist.append(get_normalized_path(line, encode=0))
+            elif line == 'debug,0':
+                debug = 0
+            elif line == 'debug,1':
+                debug = 1
+            elif line == 'error,0':
+                error = 0
+            elif line == 'error,1':
+                error = 1
+            elif line == 'propertyhandler,0':
+                pass
+            elif line == 'propertyhandler,1':
+                pass
+            else:
+                logging.debug("misaka.py: unknown directive '{0}'".format(line))
     return filelist, debug, error
 
 ###   LEXER   ###
@@ -132,7 +135,7 @@ TOKEN_DIRECTIVE     = 14
 TOKEN_TEXT          = 15
 
 
-class Lexer:
+class Lexer(object):
 
     re_comment = re.compile(r'[ \t]*//[^\r\n]*')
     re_newline = re.compile(r'\r\n|\r|\n')
@@ -170,9 +173,8 @@ class Lexer:
         TOKEN_TEXT:          'text',
         }
 
-    def __init__(self, charset='Shift_JIS', debug=0):
+    def __init__(self, charset='Shift_JIS'):
         self.buffer = []
-        self.debug = debug
         self.charset = charset
 
     def _match(self, data, line, column, path):
@@ -191,7 +193,7 @@ class Lexer:
                     lexeme = match.group()
                     if token == TOKEN_TEXT and \
                        lexeme.startswith('"') and lexeme.endswith('"'):
-                        if "$" not in lexeme:
+                        if '$' not in lexeme:
                             self.buffer.append((token, lexeme[1:-1], (line, column)))
                         else: # XXX
                             line, column = self._match(lexeme[1:-1], line, column + 1, path)
@@ -278,12 +280,11 @@ NODE_MUL_EXPR    = 13
 NODE_POW_EXPR    = 14
 
 
-class Parser:
+class Parser(object):
 
-    def __init__(self, charset, debug=0):
-        self.lexer = Lexer(charset=charset, debug=debug)
+    def __init__(self, charset):
+        self.lexer = Lexer(charset=charset)
         self.charset = charset
-        self.debug = debug
 
     def read(self, f, path=None):
         self.lexer.read(f, path)
@@ -410,17 +411,15 @@ class Parser:
             else:
                 try:
                     line = self.get_line() # beginning with brace expression
-                except MisakaError, error:
-                    if self.debug & 4:
-                        print error
+                except MisakaError as error:
+                    logging.debug(error)
                     self.lexer.skip_line()
                     return None
         else:
             try:
                 line = self.get_line()
-            except MisakaError, error:
-                if self.debug & 4:
-                    print error
+            except MisakaError as error:
+                logging.debug(error)
                 self.lexer.skip_line()
                 return None
         self.lexer.pop_check(TOKEN_NEWLINE)
@@ -792,7 +791,7 @@ class Parser:
         elif node[0] == NODE_FUNCTION:
             print ''.join((indent, 'FUNCTION')), node[1]
             for i in range(len(node[2])):
-                print ''.join((indent, 'args[%d]' % i))
+                print ''.join((indent, 'args[{0:d}]'.format(i)))
                 self.dump_list(node[2][i], depth + 1)
         elif node[0] == NODE_IF:
             print ''.join((indent, 'IF'))
@@ -894,9 +893,10 @@ class Parser:
 # unary_op   := "+" | "-"
 # factor     := OPEN_PAREN sp add_expr sp CLOSE_PAREN | word
 
+
 ###   INTERPRETER   ###
 
-class Group:
+class Group(object):
 
     def __init__(self, misaka, name, item_list=None):
         self.misaka = misaka
@@ -927,7 +927,7 @@ class NonOverlapGroup(Group):
         if not self.list:
             return None
         if not self.indexes:
-            self.indexes = range(len(self.list))
+            self.indexes = list(range(len(self.list)))
         index = self.indexes.pop(random.choice(range(len(self.indexes))))
         return self.list[index]
 
@@ -941,7 +941,7 @@ class SequentialGroup(Group):
         if not self.list:
             return None
         if not self.indexes:
-            self.indexes = range(len(self.list))
+            self.indexes = list(range(len(self.list)))
         index = self.indexes.pop(0)
         return self.list[index]
 
@@ -976,13 +976,12 @@ TYPE_SCHOLAR = 1
 TYPE_ARRAY   = 2
 
 
-class Shiori:
+class Shiori(object):
 
     DBNAME = 'misaka.db'
 
-    def __init__(self, dll_name, debug=0):
+    def __init__(self, dll_name):
         self.dll_name = dll_name
-        self.debug = debug
         self.charset = 'Shift_JIS'
 
     def use_saori(self, saori):
@@ -997,11 +996,11 @@ class Shiori:
         return result
 
     def show_description(self):
-        sys.stdout.write(
+        logging.info(
             'Shiori: MISAKA compatible module for ninix\n'
             '        Copyright (C) 2002 by Tamito KAJIYAMA\n'
             '        Copyright (C) 2002, 2003 by MATSUMURA Namihiko\n'
-            '        Copyright (C) 2002-2011 by Shyouzou Sugitani\n')
+            '        Copyright (C) 2002-2011 by Shyouzou Sugitani')
 
     def reset(self):
         self.dict = {}
@@ -1022,15 +1021,12 @@ class Shiori:
         self.saori_library = SaoriLibrary(self.saori, self.misaka_dir)
         self.reset()
         try:
-            filelist, debug, error = read_misaka_ini(self.misaka_dir,
-                                                     self.debug)
+            filelist, debug, error = read_misaka_ini(self.misaka_dir)
         except IOError:
-            if self.debug & 4:
-                print 'cannot read misaka.ini'
+            logging.debug('cannot read misaka.ini')
             return 0
-        except MisakaError, error:
-            if self.debug & 4:
-                print error
+        except MisakaError as error:
+            logging.debug(error)
             return 0
         self.misaka_debug = debug
         self.misaka_error = error
@@ -1044,11 +1040,10 @@ class Shiori:
                 try:
                     f = open(path)
                 except IOError:
-                    if self.debug & 4:
-                        print 'cannot read', filename
+                    logging.debug('cannot read {0}'.format(filename))
                     continue
-                basename, suffix = os.path.splitext(filename)
-                if suffix == '.__1':
+                basename, ext = os.path.splitext(filename)
+                if ext == '.__1':
                     f = StringIO.StringIO(self.crypt(f.read()))
                 detector.reset()
                 for line in f:
@@ -1064,17 +1059,15 @@ class Shiori:
             try:
                 f = open(path)
             except IOError:
-                if self.debug & 4:
-                    print 'cannot read', filename
+                logging.debug('cannot read {0}'.format(filename))
                 continue
-            basename, suffix = os.path.splitext(filename)
-            if suffix == '.__1':
+            basename, ext = os.path.splitext(filename)
+            if ext == '.__1':
                 f = StringIO.StringIO(self.crypt(f.read()))
             try:
                 variables, constants = self.read(f, path)
-            except MisakaError, error:
-                if self.debug & 4:
-                    print error
+            except MisakaError as error:
+                logging.debug(error)
                 continue
             global_variables.extend(variables)
             global_constants.extend(constants)
@@ -1086,7 +1079,7 @@ class Shiori:
         return 1
 
     def crypt(self, data):
-        return ''.join(map(lambda c: chr(ord(c) ^ 0xff), data))
+        return ''.join([chr(ord(c) ^ 0xff) for c in data])
 
     def eval_globals(self, sentences, constant):
         for sentence in sentences:
@@ -1097,7 +1090,7 @@ class Shiori:
                     self.eval_function(node[1], node[2])
 
     def read(self, f, path=None):
-        parser = Parser(self.charset, self.debug)
+        parser = Parser(self.charset)
         parser.read(f, path)
         common, dic = parser.get_dict()
         variables = []
@@ -1141,92 +1134,89 @@ class Shiori:
 
     def load_database(self):
         try:
-            f = open(self.dbpath)
+            with open(self.dbpath) as f:
+                while 1:
+                    line = f.readline()
+                    if not line:
+                        break
+                    header = self.strip_newline(line).split()
+                    if header[0] == 'SCHOLAR':
+                        try:
+                            name = unicode(header[1], 'utf-8')
+                        except UnicodeError:
+                            try:
+                                name = unicode(header[1], 'EUC-JP')
+                            except UnicodeError:
+                                logging.debug(
+                                    'misaka.py: malformed database (ignored)')
+                                break
+                        value = self.strip_newline(f.readline())
+                        try:
+                            value = unicode(value, 'utf-8')
+                        except UnicodeError:
+                            try:
+                                value = unicode(value, 'EUC-JP')
+                            except UnicodeError:
+                                logging.debug(
+                                    'misaka.py: malformed database (ignored)')
+                                break
+                        self.variable[name] = (TYPE_SCHOLAR, value)
+                    elif header[0] == 'ARRAY':
+                        try:
+                            size = int(header[1])
+                        except ValueError:
+                            logging.debug(
+                                'misaka.py: malformed database (ignored)')
+                            break
+                        try:
+                            name = unicode(header[2], 'utf-8')
+                        except UnicodeError:
+                            try:
+                                name = unicode(header[2], 'EUC-JP')
+                            except UnicodeError:
+                                logging.debug(
+                                    'misaka.py: malformed database (ignored)')
+                                break
+                        array = Array(self, name)
+                        for _ in range(size):
+                            value = self.strip_newline(f.readline())
+                            try:
+                                value = unicode(value, 'utf-8')
+                            except UnicodeError:
+                                try:
+                                    value = unicode(value, 'EUC-JP')
+                                except UnicodeError:
+                                    logging.debug(
+                                        'misaka.py: malformed database (ignored)')
+                                    break
+                            array.append(value)
+                        self.variable[name] = (TYPE_ARRAY, array)
+                    else:
+                        raise RuntimeError, 'should not reach here'
         except IOError:
             return
-        while 1:
-            line = f.readline()
-            if not line:
-                break
-            header = self.strip_newline(line).split()
-            if header[0] == 'SCHOLAR':
-                try:
-                    name = unicode(header[1], 'utf-8')
-                except UnicodeError:
-                    try:
-                        name = unicode(header[1], 'EUC-JP')
-                    except UnicodeError:
-                        if self.debug & 4:
-                            print 'misaka.py: malformed database (ignored)'
-                        break
-                value = self.strip_newline(f.readline())
-                try:
-                    value = unicode(value, 'utf-8')
-                except UnicodeError:
-                    try:
-                        value = unicode(value, 'EUC-JP')
-                    except UnicodeError:
-                        if self.debug & 4:
-                            print 'misaka.py: malformed database (ignored)'
-                        break
-                self.variable[name] = (TYPE_SCHOLAR, value)
-            elif header[0] == 'ARRAY':
-                try:
-                    size = int(header[1])
-                except ValueError:
-                    if self.debug & 4:
-                        print 'misaka.py: malformed database (ignored)'
-                    break
-                try:
-                    name = unicode(header[2], 'utf-8')
-                except UnicodeError:
-                    try:
-                        name = unicode(header[2], 'EUC-JP')
-                    except UnicodeError:
-                        if self.debug & 4:
-                            print 'misaka.py: malformed database (ignored)'
-                        break
-                array = Array(self, name)
-                for _ in range(size):
-                    value = self.strip_newline(f.readline())
-                    try:
-                        value = unicode(value, 'utf-8')
-                    except UnicodeError:
-                        try:
-                            value = unicode(value, 'EUC-JP')
-                        except UnicodeError:
-                            if self.debug & 4:
-                                print 'misaka.py: malformed database (ignored)'
-                            break
-                    array.append(value)
-                self.variable[name] = (TYPE_ARRAY, array)
-            else:
-                raise RuntimeError, 'should not reach here'
-        f.close()
 
     def save_database(self):
         try:
-            f = open(self.dbpath, 'w')
+            with open(self.dbpath, 'w') as f:
+                for name in self.variable:
+                    value_type, value = self.variable[name]
+                    if value_type == TYPE_SCHOLAR:
+                        if value == '':
+                            continue
+                        f.write(
+                            'SCHOLAR {0}\n{1}\n'.format(
+                                name.encode('utf-8'), unicode(value).encode('utf-8')))
+                    elif value_type == TYPE_ARRAY:
+                        f.write(
+                            'ARRAY {0:d} {1}\n'.format(len(value), name.encode('utf-8')))
+                        for item in value:
+                            f.write('{0}\n'.format(self.expand(item).encode('utf-8')))
+                    else:
+                        raise RuntimeError, 'should not reach here'
         except IOError:
-            if self.debug & 4:
-                print 'misaka.py: cannot write database (ignored)'
+            logging.debug('misaka.py: cannot write database (ignored)')
             return
-        for name in self.variable:
-            value_type, value = self.variable[name]
-            if value_type == TYPE_SCHOLAR:
-                if value == '':
-                    continue
-                f.write(
-                    'SCHOLAR %s\n%s\n' % \
-                    (name.encode('utf-8'), unicode(value).encode('utf-8')))
-            elif value_type == TYPE_ARRAY:
-                f.write(
-                    'ARRAY %d %s\n' % (len(value), name.encode('utf-8')))
-                for item in value:
-                    f.write('%s\n' % self.expand(item).encode('utf-8'))
-            else:
-                raise RuntimeError, 'should not reach here'
-        f.close()
 
     def unload(self):
         self.save_database()
@@ -1240,8 +1230,8 @@ class Shiori:
         self.req_header = {}
 
     def request(self, req_string):
-        header = StringIO.StringIO(req_string)
-        line = header.readline()
+        header = req_string.splitlines()
+        line = header.pop(0)
         if line:
             line = line.strip()
             req_list = line.split()
@@ -1254,11 +1244,12 @@ class Shiori:
                     continue
                 if ':' not in line:
                     continue
-                key, value = [x.strip() for x in line.split(':', 1)]
+                key, value = line.split(':', 1)
+                key = key.strip()
                 try:
-                    value = int(value)
+                    value = int(value.strip())
                 except:
-                    value = unicode(str(value), self.charset, 'ignore')
+                    value = unicode(str(value), self.charset, 'ignore').strip()
                 self.req_key.append(key)
                 self.req_header[key] = value
         # FIXME
@@ -1312,13 +1303,14 @@ class Shiori:
         self.reset_request()
         response = 'SHIORI/3.0 200 OK\r\n' \
                    'Sender: Misaka\r\n' \
-                   'Value: %s\r\n' % script.encode(self.charset, 'ignore')
+                   'Charset: {0}\r\n' \
+                   'Value: {1}\r\n'.format(self.charset, script.encode(self.charset, 'ignore'))
         to = self.eval_variable([[NODE_TEXT, '$to']])
         if to:
             del self.variable['$to']
             response = ''.join((response,
-                                'Reference0: %s\r\n' % \
-                                to.encode(self.charset, 'ignore')))
+                                'Reference0: {0}\r\n'.format(
+                        to.encode(self.charset, 'ignore'))))
         response = ''.join((response, '\r\n'))
         return response
 
@@ -1633,7 +1625,7 @@ class Shiori:
         return function(self, args)
 
     def is_number(self, s):
-        return s and filter(lambda c: c in '0123456789', s) == s
+        return s and all([bool(c in '0123456789') for c in s])
 
     def split(self, s):
         buf = []
@@ -1706,7 +1698,7 @@ class Shiori:
         return self.eval_variable([[NODE_TEXT, random.choice(keylist)]])
 
     def keys(self):
-        buf = self.dict.keys()
+        buf = list(self.dict.keys())
         buf.extend(self.variable.keys())
         buf.extend(self.constant.keys())
         return buf
@@ -2019,17 +2011,17 @@ class Shiori:
         req = 'EXECUTE SAORI/1.0\r\n' \
               'Sender: MISAKA\r\n' \
               'SecurityLevel: local\r\n' \
-              'Charset: %s\r\n' % self.charset
+              'Charset: {0}\r\n'.format(self.charset)
         for i in range(1, len(args)):
             req = ''.join(
                 (req,
-                 'Argument%s: %s\r\n' % \
-                 (i,
-                  self.expand_args(args[i]).encode(self.charset, 'ignore'))))
+                 'Argument{0}: {1}\r\n'.format(
+                        i,
+                        self.expand_args(args[i]).encode(self.charset, 'ignore'))))
         req = ''.join((req, '\r\n'))
         response = self.saori_library.request(self.expand_args(args[0]), req)
-        header = StringIO.StringIO(response)
-        line = header.readline()
+        header = response.splitlines()
+        line = header.pop(0)
         if line:
             line = line.strip()
             if ' ' in line:
@@ -2109,7 +2101,8 @@ class Shiori:
         '$unloadsaori':         unload_saori,
         }
 
-class SaoriLibrary:
+
+class SaoriLibrary(object):
 
     def __init__(self, saori, top_dir):
         self.saori_list = {}
@@ -2133,7 +2126,7 @@ class SaoriLibrary:
                 self.saori_list[name].unload()
                 del self.saori_list[name]
         else:
-            for saori in self.saori_list.itervalues():
+            for saori in self.saori_list.values():
                 saori.unload()
         return None
 
@@ -2145,8 +2138,8 @@ class SaoriLibrary:
         return result
 
 
-def misaka_open(top_dir, debug=0):
-    misaka = Shiori(top_dir, debug)
+def misaka_open(top_dir):
+    misaka = Shiori(top_dir)
     misaka.load()
     return misaka
 
@@ -2155,7 +2148,7 @@ def misaka_open(top_dir, debug=0):
 def test_ini(dirlist):
     for top_dir in dirlist:
         print 'Reading', os.path.join(top_dir, 'misaka.ini'), '...'
-        filelist, debug, error = read_misaka_ini(top_dir, 4)
+        filelist, debug, error = read_misaka_ini(top_dir)
         print 'number of dictionaries =', len(filelist)
         for filename in filelist:
             print filename
@@ -2163,7 +2156,7 @@ def test_ini(dirlist):
         print 'error =', error
 
 def test_lexer(filelist, charset='Shift_JIS'):
-    lexer = Lexer(charset=charset, debug=1)
+    lexer = Lexer(charset=charset)
     for filename in filelist:
         print 'Reading', filename, '...'
         lexer.read(open(filename))
@@ -2184,7 +2177,7 @@ def test_lexer(filelist, charset='Shift_JIS'):
         TOKEN_TEXT:          'TOKEN_TEXT',
         }
     for token, lexeme, position in lexer.buffer:
-        print 'L%d : C%-3d :' % position, token_names[token], ':',
+        print 'L{0:d} : C{1:-3d} :'.format(*position), token_names[token], ':',
         if token in [TOKEN_WHITESPACE, TOKEN_NEWLINE,
                      TOKEN_OPEN_BRACE, TOKEN_CLOSE_BRACE,
                      TOKEN_OPEN_PAREN, TOKEN_CLOSE_PAREN,
@@ -2198,7 +2191,7 @@ def test_lexer(filelist, charset='Shift_JIS'):
 def test_parser(filelist, charset='Shift_JIS'):
     for filename in filelist:
         print 'Reading', filename, '...'
-        parser = Parser(charset=charset, debug=1)
+        parser = Parser(charset=charset)
         parser.read(open(filename))
         common, dic = parser.get_dict()
         if common is not None:
@@ -2218,7 +2211,7 @@ def test_parser(filelist, charset='Shift_JIS'):
                     parser.dump_list(sentence, depth=2)
 
 def test_interpreter(top_dir):
-    misaka = Shiori(top_dir, debug=1)
+    misaka = Shiori(top_dir)
     misaka.load()
     while 1:
         try:
@@ -2242,6 +2235,8 @@ def test_interpreter(top_dir):
             print '  reload  reload dictionaries'
 
 if __name__ == '__main__':
+    logger = logging.getLogger()
+    logger.setLevel(logging.DEBUG) # XXX
     USAGE= 'Usage: misaka.py [ini|lexer|parser|interp] ...'
     charset = 'Shift_JIS' # XXX
     if len(sys.argv) == 1:
