@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  saori_cpuid.py - a saori_cpuid compatible Saori module for ninix
-#  Copyright (C) 2003-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2003-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -11,9 +11,8 @@
 #  PURPOSE.  See the GNU General Public License for more details.
 #
 
-import os
-import sys
 import random
+import platform
 
 from ninix.dll import SAORI
 
@@ -50,16 +49,16 @@ class Saori(SAORI):
             return self.RESPONSE[204]
         value = ''
         if argument[0] == 'platform':
-            value = sys.platform ## os.name
+            value = 'ninix-aya'
         elif argument[0] == 'os.name':
-            value = os.uname()[0]
+            value = platform.system()
         elif argument[0] == 'os.version':
-            value = os.uname()[2]
+            value = platform.release()
         elif argument[0] == 'os.build':
-            value = os.uname()[3]
+            value = platform.version()
         elif argument[0] in self.entry: ## FIXME
             value = random.choice(self.entry[argument[0]])
         if value:
-            return 'SAORI/1.0 200 OK\r\nResult: %s\r\n\r\n' % value
+            return 'SAORI/1.0 200 OK\r\nResult: {0}\r\n\r\n'.format(value)
         else:
             return self.RESPONSE[204]
