@@ -14,9 +14,8 @@
 # TODO:
 # - if, switch等の引数計算(calcを使用)
 
-import sys
-import StringIO
 import codecs
+import logging
 import re
 
 from ninix.dll import SAORI
@@ -71,12 +70,11 @@ class Saori(SAORI):
             self.value = []
             result = self.function[name][0](argument, charset)
             if result is not None and result != '':
-                s = 'SAORI/1.0 200 OK\r\nResult: %s\r\n' % result
+                s = 'SAORI/1.0 200 OK\r\nResult: {0}\r\n'.format(result)
                 if self.value:
                     for i in range(len(self.value)):
-                        s = ''.join((s,
-                                     'Value%d: %s\r\n' % (i, self.value[i])))
-                s = ''.join((s, 'Charset: %s\r\n' % charset))
+                        s = ''.join((s, 'Value{0:d}: {1}\r\n'.format(i, self.value[i])))
+                s = ''.join((s, 'Charset: {0}\r\n'.format(charset)))
                 s = ''.join((s, '\r\n'))
                 return s
             else:
@@ -85,24 +83,15 @@ class Saori(SAORI):
             return self.RESPONSE[400]
 
     def ssu_is_empty(self, args, charset):
-        if not args:
-            return 1
-        else:
-            return 0
+        return 1 if not args else 0
 
     def ssu_is_digit(self, args, charset):
         s = unicode(args[0], charset, 'ignore')
-        if s.isdigit():
-            return 1
-        else:
-            return 0
+        return 1 if s.isdigit() else 0
 
     def ssu_is_alpha(self, args, charset):
         s = unicode(args[0], charset, 'ignore')
-        if s.isalpha():
-            return 1
-        else:
-            return 0
+        return 1 if s.isalpha() else 0
 
     def ssu_length(self, args, charset):
         s = unicode(args[0], charset, 'ignore')
@@ -200,10 +189,7 @@ class Saori(SAORI):
             result = self.eval_condition(left, ope, right)
             if result:
                 return args[1]
-        if len(args) == 3:
-            return args[2]
-        else:
-            return ''
+        return args[2] if len(args) == 3 else ''
 
     def ssu_unless(self, args, charset):
         condition = unicode(
@@ -216,10 +202,7 @@ class Saori(SAORI):
             result = self.eval_condition(left, ope, right)
             if not result:
                 return args[1]
-        if len(args) == 3:
-            return args[2]
-        else:
-            return ''
+        return args[2] if len(args) == 3 else ''
 
     def ssu_iflist(self, args, charset):
         left = unicode(args[0], charset, 'ignore').encode('utf-8', 'ignore')
@@ -251,26 +234,17 @@ class Saori(SAORI):
         return args[0].count(args[1])
 
     def ssu_compare(self, args, charset):
-        if args[0] == args[1]:
-            return 1
-        else:
-            return 0
+        return 1 if args[0] == args[1] else 0
 
     def ssu_compare_head(self, args, charset):
         s0 = args[0]
         s1 = args[1]
-        if s1.startswith(s0):
-            return 1
-        else:
-            return 0
+        return 1 if s1.startswith(s0) else 0
 
     def ssu_compare_tail(self, args, charset):
         s0 = args[0]
         s1 = args[1]
-        if s1.endswith(s0):
-            return 1
-        else:
-            return 0
+        return 1 if s1.endswith(s0) else 0
 
     def ssu_erase(self, args, charset):
         return args[0].replace(args[1], '')
@@ -328,8 +302,8 @@ class Saori(SAORI):
         req_type = None
         argument = []
         charset = 'Shift_JIS' # default
-        header = StringIO.StringIO(req)
-        line = header.readline()
+        header = req.splitlines()
+        line = header.pop(0)
         if not line:
             return req_type, argument, charset
         line = line.strip()
@@ -351,7 +325,7 @@ class Saori(SAORI):
                 try:
                     codecs.lookup(charset)
                 except:
-                    sys.stderr.write('Unsupported charset %s' % repr(charset))
+                    logging.warning('Unsupported charset {0}'.format(repr(charset)))
             if key.startswith('Argument'): ## FIXME
                 argument.append(value)
             else:
