@@ -12,12 +12,7 @@
 #  PURPOSE.  See the GNU General Public License for more details.
 #
 
-import os
-
-if 'DISPLAY' in os.environ:
-    import gtk
-else:
-    gtk = None
+import gtk
 
 from ninix.dll import SAORI
 
@@ -28,9 +23,6 @@ class Saori(SAORI):
         SAORI.__init__(self)
         self.clipboard = None
 
-    def check_import(self):
-        return int(gtk is not None)
-
     def setup(self):
         self.clipboard = gtk.Clipboard(selection='PRIMARY')
         return 1
@@ -46,7 +38,7 @@ class Saori(SAORI):
         self.clipboard.set_text(text)
         if len(argument) >= 2 and argument[1] != 0:
             return 'SAORI/1.0 200 OK\r\n' \
-                   'Result: %s\r\n\r\n' % \
-                   argument[0].encode(self.charset, 'replace')
+                'Result: {0}\r\n\r\n'.format(
+                argument[0].encode(self.charset, 'replace'))
         else:
             return self.RESPONSE[204]
