@@ -16,6 +16,7 @@ import logging
 import os
 import re
 import sys
+from collections import OrderedDict
 
 import ninix.config
 import ninix.alias
@@ -74,7 +75,7 @@ def get_shiori():
 
 def search_ghosts(target=None):
     home_dir = get_ninix_home()
-    ghosts = {}
+    ghosts = OrderedDict()
     if target:
         dirlist = []
         dirlist.extend(target)
@@ -110,14 +111,18 @@ def search_ghosts(target=None):
                           shiori_dll, shiori_name)
     return ghosts
 
-def search_balloons():
+def search_balloons(target=None):
     home_dir = get_ninix_home()
+    balloons = OrderedDict()
     balloon_dir = os.path.join(home_dir, 'balloon')
-    try:
-        dirlist = os.listdir(balloon_dir)
-    except OSError:
+    if target:
         dirlist = []
-    buf = {}
+        dirlist.extend(target)
+    else:
+        try:
+            dirlist = os.listdir(balloon_dir)
+        except OSError:
+            dirlist = []
     for subdir in dirlist:
         path = os.path.join(balloon_dir, subdir)
         if not os.path.isdir(path):
@@ -133,9 +138,8 @@ def search_balloons():
             continue
         else:
             balloon_info['balloon_dir'] = (subdir, ninix.config.null_config())
-        pos = 0 if desc.get('name') == 'default' else len(buf)
-        buf[subdir] = (desc, balloon_info)
-    return buf
+        balloons[subdir] = (desc, balloon_info)
+    return balloons
 
 def search_plugins():
     home_dir = get_ninix_home()
@@ -341,7 +345,7 @@ def find_surface_set(top_dir):
         inst = read_install_txt(top_dir)
         if inst:
             shell_name = inst.get('name')
-    surface_set = {}
+    surface_set = OrderedDict()
     shell_dir = os.path.join(top_dir, 'shell')
     for name, desc, subdir in find_surface_dir(shell_dir):
         surface_dir = os.path.join(shell_dir, subdir)
