@@ -174,7 +174,7 @@ class Kinoko(object):
         else:
             self.send_event('OnKinokoObjectCreate')
         self.__running = 1
-        glib.timeout_add(10, self.do_idle_tasks) # 10ms
+        glib.timeout_add(10, self.do_idle_tasks) # 10[ms]
         return 1
 
     def do_idle_tasks(self):
