@@ -25,7 +25,7 @@
 # - speak.txt
 # - katochan が無い場合の処理.(本体の方のpopup menuなども含めて)
 # - 設定ダイアログ : [会話/反応]タブ -> [SEND SSTP/1.1] or [SHIORI]
-# - 見切れ連続20s、もしくは画面内で静止20sでアニメーション記述ミスと見なし自動的に落ちる
+# - 見切れ連続20[s]、もしくは画面内で静止20[s]でアニメーション記述ミスと見なし自動的に落ちる
 # - 発言中にバルーンをダブルクリックで即閉じ
 # - @ゴースト名は#nameと#forには使えない. もし書いても無視されすべて有効になる
 # - 連続落し不可指定
@@ -165,7 +165,7 @@ class Nekoninni(object):
         self.katochan = None
         self.launch_katochan(self.katochan_list[0])
         self.__running = 1
-        glib.timeout_add(50, self.do_idle_tasks) # 50ms
+        glib.timeout_add(50, self.do_idle_tasks) # 50[ms]
         return 1
 
     def handle_request(self, event_type, event, *arglist, **argdict):
