@@ -17,16 +17,17 @@ import re
 import time
 import urllib
 import webbrowser
+import logging
 
 from xml.dom import pulldom
 
-import ninix.home
+import gtk
+import gobject
+import cairo
 
-if 'DISPLAY' in os.environ:
-    import gtk
-    import gobject
-    import cairo
-    import ninix.pix
+import ninix.home
+import ninix.pix
+import ninix.install
 
 # 注意:
 # - このURLを本ゴーストマネージャクローン以外の用途で使う場合は,
@@ -51,7 +52,7 @@ ELEMENTS = ['Name', 'SakuraName', 'KeroName', 'GhostType',
             'AnalysisError', 'InstallCount']
 
 
-class Catalog_xml:
+class Catalog_xml(object):
 
     #public methods
     def __init__(self, datadir):
@@ -73,14 +74,13 @@ class Catalog_xml:
         last_update = self.last_update
         self.last_update = time.strftime('%Y-%m-%d %H:%M:%S')
         if self.cgi:
-            priority = self.cgi.keys()
-            priority.sort()
+            priority = sorted(self.cgi.keys())
             url = self.cgi[priority[-1]][-1]
         else:
             url = MASTERLIST_URL
         try:
             f = urllib.urlopen(url, urllib.urlencode(
-                {'time': '"%s"' % last_update, 'charset': 'UTF-8'}))
+                {'time': '"{0}"'.format(last_update), 'charset': 'UTF-8'}))
         except:
             return ## FIXME
         for _ in self.import_from_fileobj(f):
@@ -91,17 +91,15 @@ class Catalog_xml:
     # private methods
     def load_MasterList(self):
         try:
-            f = open(os.path.join(self.datadir, 'MasterList.xml'), 'r')
-            for _ in self.import_from_fileobj(f):
-                pass
-            f.close()
+            with open(os.path.join(self.datadir, 'MasterList.xml'), 'r') as f:
+                for _ in self.import_from_fileobj(f):
+                    pass
         except IOError:
             return
 
     def save_MasterList(self):
-        f = open(os.path.join(self.datadir, 'MasterList.xml'), 'w')
-        self.export_to_fileobj(f)
-        f.close()
+        with open(os.path.join(self.datadir, 'MasterList.xml'), 'w') as f:
+            self.export_to_fileobj(f)
 
     def get_encoding(self, line):
         m = re.compile('<\?xml version="1.0" encoding="(.+)" \?>').search(line)
@@ -120,7 +118,7 @@ class Catalog_xml:
         try:
             codecs.lookup(encoding)
         except:
-            raise SystemExit, 'Unsupported encoding %s' % repr(encoding)
+            raise SystemExit, 'Unsupported encoding {0}'.format(repr(encoding))
         nest = 0
         new_entry = {}
         set_id = None
@@ -204,32 +202,29 @@ class Catalog_xml:
                 text = text.replace('\'', '&apos;')
                 text = text.encode('UTF-8')
             else:
-                text = '' # fileobj.write('  <%s/>\n' % key)
-            fileobj.write('  <%s>%s</%s>\n' % (key, text, key))
+                text = '' # fileobj.write('  <{0}/>\n'.format(key))
+            fileobj.write('  <{0}>{1}</{0}>\n'.format(key, text))
 
     def export_to_fileobj(self, fileobj):
         fileobj.write('<?xml version="1.0" encoding="UTF-8" ?>\n')
         fileobj.write('<GhostList>\n')
-        fileobj.write('<LastUpdate>%s</LastUpdate>\n' % self.last_update)
+        fileobj.write('<LastUpdate>{0}</LastUpdate>\n'.format(self.last_update))
         for key in ['SakuraPreviewBaseUrl', 'KeroPreviewBaseUrl',
                     'ArcMD5BaseUrl', 'NGMUpdateBaseUrl']:
             if key in self.url:
-                fileobj.write('<%s>%s</%s>\n' % (key, self.url[key], key))
+                fileobj.write('<{0}>{1}</{0}>\n'.format(key, self.url[key]))
             else:
-                fileobj.write('<%s></%s>\n' % (key, key)) # fileobj.write('<%s/>\n' % key)
-        fileobj.write('<NGMVersion>%s</NGMVersion>\n' % self.version)
-        key_list = self.cgi.keys()
-        key_list.sort()
+                fileobj.write('<{0}></{0}>\n'.format(key)) # fileobj.write('<{0}/>\n'.format(key))
+        fileobj.write('<NGMVersion>{0}</NGMVersion>\n'.format(self.version))
+        key_list = sorted(self.cgi.keys())
         key_list.reverse()
         for priority in key_list:
             for url in self.cgi[priority]:
                 fileobj.write(
-                    '<RequestCgiUrl Priority="%d">%s</RequestCgiUrl>\n' % \
-                    (priority, url))
-        ids = self.data.keys()
-        ids.sort()
+                    '<RequestCgiUrl Priority="{0:d}">{1}</RequestCgiUrl>\n'.format(priority, url))
+        ids = sorted(self.data.keys())
         for set_id in ids:
-            fileobj.write('<FileSet ID="%s">\n' % set_id)
+            fileobj.write('<FileSet ID="{0}">\n'.format(set_id))
             entry = self.data[set_id]
             self.dump_entry(entry, fileobj)
             fileobj.write('</FileSet>\n')
@@ -241,12 +236,12 @@ class Catalog(Catalog_xml):
     TYPE = ['Sakura', 'Kero']
 
     def image_filename(self, set_id, side):
-        p = '%s_%d.png' % (self.TYPE[side], set_id)
+        p = '{0}_{1:d}.png'.format(self.TYPE[side], set_id)
         d = os.path.join(self.datadir, p)
         return d if os.path.exists(d) else None
 
     def retrieve_image(self, set_id, side, updatehook):
-        p = '%s_%d.png' % (self.TYPE[side], set_id)
+        p = '{0}_{1:d}.png'.format(self.TYPE[side], set_id)
         d = os.path.join(self.datadir, p)
         if not os.path.exists(d):
             url = self.url['SakuraPreviewBaseUrl'] if side == 0 else \
@@ -257,23 +252,10 @@ class Catalog(Catalog_xml):
                 return ## FIXME
 
 
-#class InstallationManager(list):
-#
-#    def __init__(self, ngm):
-#        self.master = ngm
-#
-#    def set_current_ghost(self, set_id):
-#        self.current_ghost = set_id
-#
-#    def install_ghost(self, dialog):
-#        a = self.master.ui.catalog.get(self.current_ghost, 'ArchiveUrl')
-#        os.system("%s '%s'" % (self.master.ninix_install_cmd, a))
+class SearchDialog(object):
 
-
-class SearchDialog:
-
-    def __init__(self, callback):
-        self.callback = callback
+    def __init__(self):
+        self.request_parent = lambda *a: None # dummy
         self.dialog = gtk.Dialog()
         self.dialog.connect('delete_event', self.cancel)
         self.dialog.set_modal(True)
@@ -292,6 +274,9 @@ class SearchDialog:
         self.dialog.action_area.pack_start(button)
         self.dialog.action_area.show_all()
 
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
     def set_pattern(self, text):
         self.pattern_entry.set_text(text)
                 
@@ -308,7 +293,7 @@ class SearchDialog:
 
     def ok(self, widget, event=None):
         word = unicode(self.get_pattern(), 'utf-8')
-        self.callback['search'](word)
+        self.request_parent('NOTIFY', 'search', word)
         self.hide()
         return True
 
@@ -317,9 +302,10 @@ class SearchDialog:
         return True
 
 
-class UI:
-    def __init__(self, callback):
-        self.callback = callback
+class UI(object):
+
+    def __init__(self):
+        self.request_parent = lambda *a: None # dummy
         self.ui_info = '''
         <ui>
           <menubar name='MenuBar'>
@@ -360,7 +346,8 @@ class UI:
             ( 'Settings', None,
               _('Settings(_O)'), None,
               None,
-              lambda *a: self.callback['open_preference_dialog']() ),
+              lambda *a: self.request_parent(
+                    'NOTIFY', 'open_preference_dialog') ),
             ( 'DB Network Update', None,
               _('DB Network Update(_N)'), None,
               None,
@@ -372,15 +359,18 @@ class UI:
             ( 'Mask', None,
               _('Mask(_M)'), None,
               None,
-              lambda *a: self.callback['open_mask_dialog']() ),
+              lambda *a: self.request_parent(
+                    'NOTIFY', 'open_mask_dialog') ),
             ( 'Reset to Default', None,
               _('Reset to Default(_Y)'), None,
               None,
-              lambda *a: self.callback['reset_to_default']() ),
+              lambda *a: self.request_parent(
+                    'NOTIFY', 'reset_to_default') ),
             ( 'Show All', None,
               _('Show All(_Z)'), None,
               None,
-              lambda *a: self.callback['show_all']() ),
+              lambda *a: self.request_parent(
+                    'NOTIFY', 'show_all') ),
             )
         self.opened = 0
         self.textview = [None, None]
@@ -390,12 +380,26 @@ class UI:
         self.button = {}
         self.url = {}
         self.search_word = ''
-        search_dialog_callback = {
-            'search': self.search,
-            }
-        self.search_dialog = SearchDialog(search_dialog_callback)
+        self.search_dialog = SearchDialog()
+        self.search_dialog.set_responsible(self.handle_request)
         self.create_dialog()
 
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
+    def handle_request(self, event_type, event, *arglist, **argdict):
+        assert event_type in ['GET', 'NOTIFY']
+        handlers = {
+            }
+        handler = handlers.get(event, getattr(self, event, None))
+        if handler is None:
+            result = self.request_parent(
+                event_type, event, *arglist, **argdict)
+        else:
+            result = handler(*arglist, **argdict)
+        if event_type == 'GET':
+            return result
+
     def create_dialog(self):
         self.window = gtk.Window()
         self.window.set_title(_('Ghost Manager'))
@@ -410,8 +414,8 @@ class UI:
         self.window.add_accel_group(ui.get_accel_group())
         try:
             mergeid = ui.add_ui_from_string(self.ui_info)
-        except gobject.GError, msg:
-            print 'building menus failed: %s' % msg
+        except gobject.GError as msg:
+            logging.error('building menus failed: {0}'.format(msg))
         vbox = gtk.VBox(False, 0)
         self.window.add(vbox)
         vbox.show()
@@ -451,7 +455,7 @@ class UI:
         def updatehook(*args):
             while gtk.events_pending():
                 gtk.main_iteration()
-        self.callback['network_update'](updatehook)
+        self.request_parent('NOTIFY', 'network_update', updatehook)
         self.update()
         self.window.set_sensitive(True)
 
@@ -461,24 +465,24 @@ class UI:
     def search(self, word):
         if word:
             self.search_word = word
-            if self.callback['search'](word):
+            if self.request_parent('GET', 'search', word):
                 self.update()
             else:
                 pass ## FIXME
 
     def search_forward(self):
         if self.search_word:
-            if self.callback['search_forward'](self.search_word):
+            if self.request_parent('GET', 'search_forward', self.search_word):
                 self.update()
             else:
                 pass ## FIXME
 
     def show_next(self):
-        self.callback['next']()
+        self.request_parent('NOTIFY', 'next')
         self.update()
 
     def show_previous(self):
-        self.callback['previous']()
+        self.request_parent('NOTIFY', 'previous')
         self.update()
 
     def create_surface_area(self, side):
@@ -512,11 +516,11 @@ class UI:
 
     def update_surface_area(self):
         for side in [0, 1]:
-            name = self.callback['get']('SakuraName') if side == 0 else \
-                self.callback['get']('KeroName')
+            target = 'SakuraName' if side == 0 else 'KeroName'
+            name = self.request_parent('GET', 'get', target)
             textbuffer = self.textview[side].get_buffer()
             textbuffer.set_text(name)
-            filename = self.callback['get_image_filename'](side)
+            filename = self.request_parent('GET', 'get_image_filename', side)
             darea = self.darea[side]
             darea.realize()
             if filename is not None:
@@ -545,12 +549,16 @@ class UI:
         box.set_layout(gtk.BUTTONBOX_SPREAD)
         box.show()
         button = gtk.Button(_('Install'))
-        button.connect('clicked', lambda b, w=self: w.callback['install_current']())
+        button.connect(
+            'clicked',
+            lambda b, w=self: w.request_parent('NOTIFY', 'install_current'))
         box.add(button)
         button.show()
         self.button['install'] = button
         button = gtk.Button(_('Update'))
-        button.connect('clicked', lambda b, w=self: w.callback['update_current']())
+        button.connect(
+            'clicked',
+            lambda b, w=self: w.request_parent('NOTIFY', 'update_current'))
         box.add(button)
         button.show()
         self.button['update'] = button
@@ -591,44 +599,51 @@ class UI:
                      (_('Version:'), 'Version'),
                      (_('AIName:'), 'AIName')]
         text = ''
-        text = ''.join((text, self.callback['get']('Name'), '\n'))
+        text = ''.join((text, self.request_parent('GET', 'get', 'Name'), '\n'))
         for item in info_list:
-            text = ''.join((text, item[0], self.callback['get'](item[1]), '\n'))
+            text = ''.join((text, item[0],
+                            self.request_parent('GET', 'get', item[1]), '\n'))
         text = ''.join((text,
-                        self.callback['get']('SakuraName'),
-                        _('SurfaceList:'), self.callback['get']('SakuraSurfaceList'),
+                        self.request_parent('GET', 'get', 'SakuraName'),
+                        _('SurfaceList:'),
+                        self.request_parent('GET', 'get', 'SakuraSurfaceList'),
                         '\n'))
         text = ''.join((text,
-                        self.callback['get']('KeroName'),
-                        _('SurfaceList:'), self.callback['get']('KeroSurfaceList'),
+                        self.request_parent('GET', 'get', 'KeroName'),
+                        _('SurfaceList:'),
+                        self.request_parent('GET', 'get', 'KeroSurfaceList'),
                         '\n'))
         textbuffer = self.info.get_buffer()
         textbuffer.set_text(text)
-        url = self.callback['get']('HPUrl')
-        text = self.callback['get']('HPTitle')
+        url = self.request_parent('GET', 'get', 'HPUrl')
+        text = self.request_parent('GET', 'get', 'HPTitle')
         self.url['HP'][0] = url
         label = self.url['HP'][1]
-        label.set_markup('<span foreground="blue">%s</span>' % text)
-        url = self.callback['get']('PublicUrl')
-        text = ''.join((self.callback['get']('Name'), _(' Web Page')))
+        label.set_markup('<span foreground="blue">{0}</span>'.format(text))
+        url = self.request_parent('GET', 'get', 'PublicUrl')
+        text = ''.join((self.request_parent('GET', 'get', 'Name'),
+                        _(' Web Page')))
         self.url['Public'][0] = url
         label = self.url['Public'][1]
-        label.set_markup('<span foreground="blue">%s</span>' % text)        
-        target_dir = os.path.join(self.callback['get_home_dir'](), 'ghost',
-                                  self.callback['get']('InstallDir'))
+        label.set_markup('<span foreground="blue">{0}</span>'.format(text))
+        target_dir = os.path.join(
+            self.request_parent('GET', 'get_home_dir'), 'ghost',
+            self.request_parent('GET', 'get', 'InstallDir'))
         self.button['install'].set_sensitive(
             bool(not os.path.isdir(target_dir) and
-                 self.callback['get']('ArchiveUrl') != 'No data'))
+                 self.request_parent('GET', 'get', 'ArchiveUrl') != 'No data'))
         self.button['update'].set_sensitive(
             bool(os.path.isdir(target_dir) and
-                 self.callback['get']('GhostType') == 'ゴースト' and
-                 self.callback['get']('UpdateUrl') != 'No data'))
+                 self.request_parent('GET', 'get', 'GhostType') == 'ゴースト' and
+                 self.request_parent('GET', 'get', 'UpdateUrl') != 'No data'))
 
     def update(self):
         self.update_surface_area()
         self.update_info_area()
-        self.button['next'].set_sensitive(bool(self.callback['exist_next']()))
-        self.button['previous'].set_sensitive(bool(self.callback['exist_previous']()))
+        self.button['next'].set_sensitive(
+            bool(self.request_parent('GET', 'exist_next')))
+        self.button['previous'].set_sensitive(
+            bool(self.request_parent('GET', 'exist_previous')))
 
     def show(self):
         if self.opened:
@@ -643,33 +658,34 @@ class UI:
         return True
 
 
-class NGM:
+class NGM(object):
 
-    def __init__(self, callback):
-        self.callback = callback
+    def __init__(self):
+        self.request_parent = lambda *a: None # dummy
         self.current = 0
         self.opened = 0
         self.home_dir = ninix.home.get_ninix_home()
         self.catalog = Catalog(os.path.join(self.home_dir, 'ngm/data'))
-        ui_callback = {
-            'open_preference_dialog': self.open_preference_dialog,
-            'open_mask_dialog': self.open_mask_dialog,
-            'reset_to_default': self.reset_to_default,
-            'show_all': self.show_all,
-            'network_update': self.network_update,
-            'search': self.search,
-            'search_forward': self.search_forward,
-            'next': self.next,
-            'previous': self.previous,
-            'get': self.get,
-            'get_image_filename': self.get_image_filename,
+        self.installer = ninix.install.Installer()
+        self.ui = UI()
+        self.ui.set_responsible(self.handle_request)
+
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
+    def handle_request(self, event_type, event, *arglist, **argdict):
+        assert event_type in ['GET', 'NOTIFY']
+        handlers = {
             'get_home_dir': lambda *a: self.home_dir,
-            'exist_next': self.exist_next,
-            'exist_previous': self.exist_previous,
-            'install_current': self.install_current,
-            'update_current': self.update_current,
             }
-        self.ui = UI(ui_callback)
+        handler = handlers.get(event, getattr(self, event, None))
+        if handler is None:
+            result = self.request_parent(
+                event_type, event, *arglist, **argdict)
+        else:
+            result = handler(*arglist, **argdict)
+        if event_type == 'GET':
+            return result
 
     def get(self, element):
         if self.current in self.catalog.data:
@@ -739,11 +755,18 @@ class NGM:
     def show_dialog(self):
         self.ui.show()
 
-    def install_current(self): ## FIXME
-        pass
+    def install_current(self):
+        try:
+            filetype, target_dir = self.installer.install(
+                self.get('ArchiveUrl'), ninix.home.get_ninix_home())
+        except:
+            target_dir = None
+        assert filetype == 'ghost'
+        if target_dir is not None:
+            self.request_parent('NOTIFY', 'add_sakura', target_dir)
 
     def update_current(self): ## FIXME
-        self.callback['update_sakura'](self.get('Name'), 'NGM')
+        self.request_parent('NOTIFY', 'update_sakura', self.get('Name'), 'NGM')
 
 
 if __name__ == '__main__':
