@@ -94,7 +94,6 @@ class PreferenceDialog(object):
 
     PREFS_TYPE = {'sakura_name': None, # XXX: backward compat
                   'sakura_dir': None,
-                  'sakura_surface': None,
                   'default_balloon': None,
                   'ignore_default': int,
                   'script_speed': int,
@@ -193,7 +192,7 @@ class PreferenceDialog(object):
         assert name in self.PREFS_TYPE
         return self.__prefs.get_with_type(name, self.PREFS_TYPE[name], default)
 
-    def set_current_sakura(self, directory, surface):
+    def set_current_sakura(self, directory):
         key = 'sakura_name' # obsolete
         if key in self.__prefs:
             del self.__prefs[key]
@@ -201,10 +200,6 @@ class PreferenceDialog(object):
         if key in self.__prefs:
             del self.__prefs[key]
         self.__prefs[key] = directory
-        key = 'sakura_surface'
-        if key in self.__prefs:
-            del self.__prefs[key]
-        self.__prefs[key] = surface
 
     def edit_preferences(self):
         self.show()
