@@ -24,6 +24,7 @@ import sys
 import time
 import webbrowser
 import urllib
+from collections import OrderedDict
 
 import gtk ## FIXME
 import glib
@@ -54,14 +55,33 @@ import ninix.script
 import ninix.version
 import ninix.update
 from ninix.home import get_ninix_home, get_normalized_path
+from ninix.metamagic import Meme
+
+
+class ShellMeme(Meme):
+
+    def __init__(self, key):
+        Meme.__init__(self, key)
+        self.request_parent = lambda *a: None # dummy
+
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
+    def create_menuitem(self, data):
+        shell_name = data[0]
+        return self.request_parent(
+            'GET', 'create_shell_menuitem', shell_name, self.key)
+
+    def delete_by_myself(self):
+        self.request_parent('NOTIFY', 'delete_shell', self.key)
 
 
 class Sakura(object):
 
-    BALLOON_LIFE   = 10  # sec (0: never closed automatically)
-    SELECT_TIMEOUT = 15  # sec
-    PAUSE_TIMEOUT  = 30  # sec
-    SILENT_TIME    = 15  # sec
+    BALLOON_LIFE   = 10  # [sec] (0: never closed automatically)
+    SELECT_TIMEOUT = 15  # [sec]
+    PAUSE_TIMEOUT  = 30  # [sec]
+    SILENT_TIME    = 15  # [sec]
     # script modes
     BROWSE_MODE = 1
     SELECT_MODE = 2
@@ -75,7 +95,7 @@ class Sakura(object):
     except:
         name2codepoint = None
 
-    def __init__(self, data):
+    def __init__(self):
         self.request_parent = lambda *a: None # dummy
         self.sstp_handle = None
         self.sstp_entry_db = None
@@ -124,7 +144,6 @@ class Sakura(object):
             self.audio_player.set_property('video-sink', fakesink)
         else:
             self.audio_player = None
-        self.new(*data)
 
     def set_responsible(self, request_method):
         self.request_parent = request_method
@@ -132,8 +151,6 @@ class Sakura(object):
     def handle_request(self, event_type, event, *arglist, **argdict):
         assert event_type in ['GET', 'NOTIFY']
         handlers = {
-            'close_sakura': self.close,
-            'vanish_sakura': self.vanish,
             }
         handler = handlers.get(event, getattr(self, event, None))
         if handler is None:
@@ -144,21 +161,6 @@ class Sakura(object):
         if event_type == 'GET':
             return result
 
-    def select_sakura(self, item):
-        if self.busy():
-            gtk.gdk.beep()
-            return
-        self.request_parent('NOTIFY', 'change_sakura', self, item, 'manual')
-
-    def select_plugin(self, item):
-        self.request_parent('NOTIFY', 'select_plugin', item, self)
-
-    def select_nekodorif(self, item):
-        self.request_parent('NOTIFY', 'select_nekodorif', item, self)
-
-    def select_kinoko(self, item):
-        self.request_parent('NOTIFY', 'select_kinoko', item, self)
-
     def attach_observer(self, observer):
         if observer not in self.__observers:
             self.__observers[observer] = 1
@@ -172,14 +174,34 @@ class Sakura(object):
         if observer in self.__observers:
             del self.__observers[observer]
 
+    def delete_shell(self, key):
+        assert key in self.shells
+        del self.shells[key]
+
+    def get_shell_menu(self):
+        current_key = self.get_current_shell()
+        for key in self.shells:
+            menuitem = self.shells[key].menuitem
+            menuitem.set_sensitive(key != current_key) # not working
+        return self.shell_menu
+
     def new(self, desc, shiori_dir, use_makoto, surface_set, prefix,
             shiori_dll, shiori_name): ## FIXME
         self.shiori = None
         self.desc = desc
         self.shiori_dir = shiori_dir
         self.use_makoto = use_makoto
-        self.surface_set = surface_set
-        self.prefix = prefix ## FIXME: ninix_home should not be included.(append in get_prefix())
+        self.shells = OrderedDict()
+        shell_menuitems = OrderedDict()
+        for key, value in surface_set.items():
+            meme = ShellMeme(key)
+            meme.set_responsible(self.handle_request)
+            self.shells[key] = meme
+            meme.baseinfo = value
+            shell_menuitems[key] = meme.menuitem
+        self.shell_menu = self.request_parent(
+            'GET', 'create_shell_menu', shell_menuitems)
+        self.prefix = prefix
         self.shiori_dll = shiori_dll
         self.shiori_name = shiori_name
         name = (shiori_dll, shiori_name)
@@ -215,6 +237,9 @@ class Sakura(object):
                 if self.balloon_directory is not None:
                     f.write('balloon_directory, {0}\n'.format(
                             self.balloon_directory))
+                if self.shell_directory is not None:
+                    f.write('shell_directory, {0}\n'.format(
+                            self.shell_directory))
         except IOError as e:
             code, message = e.args
             logging.error('cannot write {0}'.format(path))
@@ -254,6 +279,7 @@ class Sakura(object):
         path = os.path.join(self.get_prefix(), 'SETTINGS')
         if os.path.exists(path):
             balloon_directory = None
+            shell_directory = None
             try:
                 with open(path, 'r') as f:
                     for line in f:
@@ -262,12 +288,16 @@ class Sakura(object):
                         key, value = line.split(',', 1)
                         if key.strip() == 'balloon_directory':
                             balloon_directory = value.strip()
+                        if key.strip() == 'shell_directory':
+                            shell_directory = value.strip()
             except IOError as e:
                 code, message = e.args
                 logging.error('cannot read {0}'.format(path))
             self.balloon_directory = balloon_directory
+            self.shell_directory = shell_directory
         else:
             self.balloon_directory = None
+            self.shell_directory = None
 
     def load_shiori(self):
         if self.shiori and self.shiori.load(self.shiori_dir):
@@ -295,8 +325,8 @@ class Sakura(object):
     def leave_temp_mode(self):
         self.__temp_mode = 0
 
-    def set_surface(self, desc, alias, surface, name, dir, tooltips):
-        self.surface.new(desc, alias, surface, name, dir, tooltips)
+    def set_surface(self, desc, alias, surface, name, surface_dir, tooltips):
+        self.surface.new(desc, alias, surface, name, surface_dir, tooltips)
         for side in range(2, self.char):
             default = self.desc.get('char{0:d}.seriko.defaultsurface'.format(side))
             self.surface.add_window(side, default)
@@ -463,17 +493,25 @@ class Sakura(object):
             lambda a: self.request_parent('NOTIFY', 'vanish_sakura', a),
             (self))
 
-    def get_name(self):
-        return self.desc.get('name', unicode(_('Sakura&Unyuu'), 'utf-8'))
+    def get_ifghost(self):
+        return '{0},{1}'.format(self.get_selfname(), self.get_keroname())
+
+    def ifghost(self, ifghost):
+        names = self.get_ifghost()
+        name = '{0}'.format(self.get_selfname())
+        return bool(ifghost in [name, names])
+
+    def get_name(self, default=unicode(_('Sakura&Unyuu'), 'utf-8')):
+        return self.desc.get('name', default)
 
     def get_username(self):
         return self.getstring('username') or \
                self.surface.get_username() or \
                self.desc.get('user.defaultname', unicode(_('User'), 'utf-8'))
 
-    def get_selfname(self):
+    def get_selfname(self, default=unicode(_('Sakura'), 'utf-8')):
         return self.surface.get_selfname() or \
-               self.desc.get('sakura.name', unicode(_('Sakura'), 'utf-8'))
+               self.desc.get('sakura.name', default)
 
     def get_selfname2(self):
         return self.surface.get_selfname2() or \
@@ -500,7 +538,7 @@ class Sakura(object):
         result = self.get_event_response('{0}'.format(word_type))
         return self.translate(result)
 
-    def getstring(self, name): ## FIXME
+    def getstring(self, name):
         return self.get_event_response(name)
 
     def translate(self, s):
@@ -794,7 +832,8 @@ class Sakura(object):
             if event in self.boot_event:
                 self.surface_bootup()
             if event == 'OnMouseClick' and arglist[5] == 3:
-                self.surface.open_popup_menu(arglist[5], arglist[3])
+                self.request_parent(
+                    'NOTIFY', 'open_popup_menu', self, arglist[5], arglist[3])
             return 0
         logging.debug('=> "{0}"'.format(script.encode('utf-8', 'ignore')))
         if self.__temp_mode == 2:
@@ -814,37 +853,61 @@ class Sakura(object):
     def get_prefix(self):
         return self.prefix
 
-    def get_shell_list(self):
-        shell_list = []
-        set_type, i, j = self.current
-        assert set_type == 'g'
-        keys = sorted(self.surface_set.keys())
-        for key in keys:
-            name = self.surface_set[key][0]
-            value = (set_type, i, key)
-            working = 1 if key == j else 0 
-            shell_list.append((name, value, working))
-        return shell_list
+    def stick_window(self, flag):
+        self.surface.window_stick(flag)
+
+    def toggle_bind(self, args):
+        self.surface.toggle_bind(args)
+
+    def get_menu_pixmap(self):
+        path_background, path_sidebar, path_foreground = \
+            self.surface.get_menu_pixmap()
+        top_dir = os.path.join(self.get_prefix(), 'ghost', 'master')
+        if not os.path.exists(path_background):
+            path = os.path.join(top_dir, 'menu_background.png')
+            if os.path.exists(path):
+                path_background = path
+            else:
+                path_background = None
+        if not os.path.exists(path_sidebar):
+            path = os.path.join(top_dir, 'menu_sidebar.png')
+            if os.path.exists(path):
+                path_sidebar = path
+            else:
+                path_sidebar = None
+        if not os.path.exists(path_foreground):
+            path = os.path.join(top_dir, 'menu_foreground.png')
+            if os.path.exists(path):
+                path_foreground = path
+            else:
+                path_foreground = None
+        return path_background, path_sidebar, path_foreground
+
+    def get_menu_fontcolor(self):
+        return self.surface.get_menu_fontcolor()
+
+    def get_mayuna_menu(self):
+        return self.surface.get_mayuna_menu()
 
     def get_current_balloon_directory(self):
         return self.balloon.get_balloon_directory()
 
-    def select_shell(self, item):
-        set_type, i, j = item
-        assert self.current[0] == set_type
-        assert self.current[1] == i
-        assert self.current[2] != j
-        assert set_type == 'g'
-        assert self.surface_set and j in self.surface_set
-        name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
-            self.surface_set[j]
-        if not name: ## FIXME
-            surface_name = j
-        else:
-            surface_name = name
-        def proc(self=self, item=item):
-            logging.info('ghost {0} {1} {2}'.format(*item))
-            self.current = item
+    def get_current_shell(self):
+        return self.shell_directory
+
+    def get_default_shell(self):
+        default = self.shell_directory or 'master'            
+        if default not in self.shells:
+            default = self.shells.keys()[0] # XXX
+        return default
+
+    def select_shell(self, shell_key):
+        assert self.shells and shell_key in self.shells
+        self.shell_directory = shell_key # save user's choice
+        surface_name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
+            self.shells[shell_key].baseinfo
+        def proc(self=self, key=shell_key):
+            logging.info('ghost {0} {1}'.format(self.key, key))
             self.set_surface(surface_desc, surface_alias, surface, surface_name,
                              surface_dir, surface_tooltips)
             self.surface.reset_alignment()
@@ -854,22 +917,14 @@ class Sakura(object):
         self.enqueue_event('OnShellChanging', surface_name, surface_dir,
                            proc=proc)
 
-    def select_balloon(self, widget, item):
-        if not widget.active:
-            return
+    def select_balloon(self, item, desc, balloon):
+        self.balloon_directory = item # save user's choice
         if item == self.get_current_balloon_directory(): # no change
-            # need reloadning?
-            if self.balloon_directory != item:
-                self.balloon_directory = item # save user's choice
-            return
-        desc, balloon = self.request_parent(
-            'GET', 'get_balloon_description', item)
-        subdir = balloon['balloon_dir'][0]
-        path = os.path.join(get_ninix_home(), 'balloon', subdir)
+            return # need reloadning?
+        assert item == balloon['balloon_dir'][0]
+        path = os.path.join(get_ninix_home(), 'balloon', item)
         self.balloon.hide_all()
         self.set_balloon(desc, balloon)
-        if item == subdir:
-            self.balloon_directory = subdir # save user's choice
         self.balloon.set_balloon_default()
         self.position_balloons()
         name = desc.get('name', '')
@@ -999,7 +1054,7 @@ class Sakura(object):
             self.__surface_life = random.randint(20, 30)
             ##logging.debug('surface_life = {0:d}'.format(self.__surface_life))
 
-    def start(self, item, init, temp, vanished, ghost_changed, prev_name):
+    def start(self, key, init, temp, vanished, ghost_changed, prev_name):
         if self.is_running():
             if temp:
                 self.enter_temp_mode()
@@ -1014,46 +1069,35 @@ class Sakura(object):
         self.vanished_count = 0
         self.__running = 1
         self.__temp_mode = temp
-        self.current = item
-        set_type, i, j = item
-        assert set_type == 'g'
-        logging.info('ghost {0} {1} {2}'.format(*item))
-        assert self.surface_set and j in self.surface_set
-        name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
-            self.surface_set[j]
-        surface_set = self.surface_set
-        if not name: ## FIXME
-            surface_name = j
-        else:
-            surface_name = name
-        if ghost_changed:
-            name = prev_name
+        self.key = key
+        logging.info('ghost {0}'.format(key))
+        self.load_settings()
+        shell_key = self.get_default_shell()
+        self.shell_directory = shell_key # XXX
+        assert self.shells and shell_key in self.shells
+        surface_name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
+            self.shells[shell_key].baseinfo
+        name = prev_name if ghost_changed else surface_name
         self.set_surface(surface_desc, surface_alias, surface, surface_name,
                          surface_dir, surface_tooltips)
-        self.load_settings()
-        if self.request_parent('GET', 'get_preference', 'ignore_default'): ## FIXME: change prefs key
-            default_balloon = self.request_parent(
-                'GET', 'get_preference', 'default_balloon')
-            balloon = self.request_parent(
-                'GET', 'get_balloon_description', default_balloon)
-        elif self.balloon_directory is not None:
-            balloon = self.request_parent(
-                'GET', 'get_balloon_description', self.balloon_directory)
-        else:
+        balloon = None
+        if not self.request_parent('GET', 'get_preference', 'ignore_default'): ## FIXME: change prefs key
             balloon_path = self.desc.get('deault.balloon.path', '')
             balloon_name = self.desc.get('balloon', '')
-            balloon = None
             if balloon_path:
                 balloon = self.request_parent(
                     'GET', 'find_balloon_by_subdir', balloon_path)
             if balloon is None and balloon_name:
                 balloon = self.request_parent(
                     'GET', 'find_balloon_by_name', balloon_name)
-            if balloon is None:
+        if balloon is None:
+            if self.balloon_directory is not None:
+                default_balloon = self.balloon_directory
+            else:
                 default_balloon = self.request_parent(
                     'GET', 'get_preference', 'default_balloon')
-                balloon = self.request_parent(
-                    'GET', 'get_balloon_description', default_balloon)
+            balloon = self.request_parent(
+                'GET', 'get_balloon_description', default_balloon)
         desc, balloon = balloon
         self.set_balloon(desc, balloon)
         if not temp:
@@ -1061,7 +1105,7 @@ class Sakura(object):
         self.restart()
         self.start_time = time.time()
         self.notify_start(init, vanished, ghost_changed, name, surface_dir)
-        glib.timeout_add(10, self.do_idle_tasks) # 10ms
+        glib.timeout_add(10, self.do_idle_tasks) # 10[ms]
 
     def restart(self):
         self.load_history()
@@ -1406,13 +1450,13 @@ class Sakura(object):
         script_speed = self.request_parent(
             'GET', 'get_preference', 'script_speed')
         if not self.quick_session and script_speed >= 0:
-            self.__set_weight(args[0], 0.05) # 50ms
+            self.__set_weight(args[0], 0.05) # 50[ms]
 
     def __yen__w(self, args):
         script_speed = self.request_parent(
             'GET', 'get_preference', 'script_speed')
         if not self.quick_session and script_speed >= 0:
-            self.__set_weight(args[0], 0.001) # 1ms
+            self.__set_weight(args[0], 0.001) # 1[ms]
 
     def __yen_t(self, args):
         self.time_critical_session = not self.time_critical_session
@@ -1565,13 +1609,10 @@ class Sakura(object):
         elif args[0:2] == ['open', 'configurationdialog']:
             self.request_parent('NOTIFY', 'edit_preferences')
         elif args[0:2] == ['change', 'shell'] and argc > 2:
-            set_type, i, j = self.current
-            assert set_type == 'g'
-            for key in self.surface_set:
-                shell_name = self.surface_set[key][0]
+            for key in self.shells:
+                shell_name = self.shells[key].baseinfo[0]
                 if shell_name == args[2]:
-                    item = (set_type, i, key)
-                    self.select_shell(item)
+                    self.select_shell(key)
                     break
         elif args[0:2] == ['change', 'ghost'] and argc > 2:
             if args[2] == 'random':
