@@ -13,37 +13,49 @@
 #
 
 import codecs
-import sys
+import logging
 import socket
+from collections import OrderedDict
 
 import ninix.entry_db
 import ninix.script
 import ninix.version
 
-from sstplib import AsynchronousSSTPServer, BaseSSTPRequestHandler
+from ninix.sstplib import AsynchronousSSTPServer, BaseSSTPRequestHandler
 
 
 class SSTPServer(AsynchronousSSTPServer):
 
-    def __init__(self, address, callback):
-        self.callback = callback
+    def __init__(self, address):
+        self.request_parent = lambda *a: None # dummy
         AsynchronousSSTPServer.__init__(self, address, SSTPRequestHandler)
         self.request_handler = None
 
+    def shutdown_request(self, request):
+        if self.request_handler is not None:
+            # XXX: send_* methods can be called from outside of the handler
+            pass # keep alive
+        else:
+            AsynchronousSSTPServer.shutdown_request(self, request)
+
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
     def send_response(self, code, data=None):
         try:
             self.request_handler.send_response(code)
             if data is not None:
                 self.request_handler.wfile.write(data)
             self.request_handler.force_finish()
+            self.socket.shutdown(socket.SHUT_WR) # XXX
         except IOError:
             pass
         self.request_handler = None
 
     def send_answer(self, value):
-        self.send_response(200, # OK
-                           ''.join((value.encode('utf-8', 'ignore'),
-                                    '\r\nCharset: UTF-8\r\n')))
+        charset = self.request_handler.headers.get('charset', 'Shift_JIS')
+        answer = ''.join((value.encode(charset, 'ignore'), '\r\n\r\n'))
+        self.send_response(200, answer) # OK
 
     def send_no_content(self):
         self.send_response(204) # No Content
@@ -61,7 +73,7 @@ class SSTPServer(AsynchronousSSTPServer):
 class SSTPRequestHandler(BaseSSTPRequestHandler):
 
     def handle(self):
-        if not self.server.callback['get_sakura_cantalk']():
+        if not self.server.request_parent('GET', 'get_sakura_cantalk'):
             self.error = self.version = None
             if not self.parse_request(self.rfile.readline()):
                 return
@@ -104,11 +116,8 @@ class SSTPRequestHandler(BaseSSTPRequestHandler):
                 return
         else:
             handle = None
-        if version == 1.4:
-            script, if_ghost = self.get_script_if_ghost()
-        else:
-            script = self.get_script()
-        if script is None or self.check_script(script):
+        script_odict = self.get_script_odict()
+        if script_odict is None:
             return
         if version in [1.0, 1.1]:
             entry_db = None
@@ -116,11 +125,7 @@ class SSTPRequestHandler(BaseSSTPRequestHandler):
             entry_db = self.get_entry_db()
             if entry_db is None:
                 return
-        if version == 1.4:
-            self.enqueue_script_if_ghost(
-                if_ghost, sender, handle, script, entry_db)
-        else:
-            self.enqueue_script(sender, handle, script, entry_db)
+        self.enqueue_request(sender, None, handle, script_odict, entry_db)
 
     # NOTIFY
     def do_NOTIFY_1_0(self):
@@ -138,37 +143,18 @@ class SSTPRequestHandler(BaseSSTPRequestHandler):
         event = self.get_event()
         if event is None:
             return
-        script = self.server.callback['get_event_response'](*event)
-        if script and self.check_script(script):
-            return
         if version == 1.0:
             entry_db = None
         elif version == 1.1:
-            if not script:
-                script, if_ghost = self.get_script_if_ghost(current=1)
-                if script is None or self.check_script(script):
-                    return
+            script_odict = self.get_script_odict()
+            if script_odict is None:
+                return
             entry_db = self.get_entry_db()
             if entry_db is None:
                 return
-        if not script:
-            self.send_response(200) # OK
-            return
-        self.enqueue_script(sender, None, script, entry_db)
+        self.enqueue_request(sender, event, None, script_odict, entry_db)
 
-    def enqueue_script_if_ghost(self, if_ghost, sender, handle,
-                                script, entry_db):
-        try:
-            address = self.client_address[0]
-        except:
-            address = self.client_address
-        self.send_response(200) # OK
-        show_sstp_marker, use_translator = self.get_options()
-        self.server.callback['enqueue_script_if_ghost'](
-            if_ghost, script, sender, handle,
-            address, show_sstp_marker, use_translator, entry_db)
-
-    def enqueue_script(self, sender, handle, script, entry_db):
+    def enqueue_request(self, sender, event, handle, script_odict, entry_db):
         try:
             address = self.client_address[0]
         except:
@@ -176,16 +162,20 @@ class SSTPRequestHandler(BaseSSTPRequestHandler):
         if entry_db is None or entry_db.is_empty():
             self.send_response(200) # OK
             show_sstp_marker, use_translator = self.get_options()
-            self.server.callback['enqueue_script'](
-                script, sender, handle, address,
-                show_sstp_marker, use_translator)
+            self.server.request_parent(
+                'NOTIFY', 'enqueue_request',
+                event, script_odict, sender, handle,
+                address, show_sstp_marker, use_translator,
+                entry_db, None)
         elif self.server.request_handler:
             self.send_response(409) # Conflict
         else:
             show_sstp_marker, use_translator = self.get_options()
-            self.server.callback['enqueue_script'](
-                script, sender, handle, address,
-                show_sstp_marker, use_translator, entry_db, self.server)
+            self.server.request_parent(
+                'NOTIFY', 'enqueue_request',
+                event, script_odict, sender, handle,
+                address, show_sstp_marker, use_translator,
+                entry_db, self.server)
             self.server.request_handler = self # keep alive
 
     PROHIBITED_TAGS = [r'\j', r'\-', r'\+', r'\_+', r'\!']
@@ -197,7 +187,7 @@ class SSTPRequestHandler(BaseSSTPRequestHandler):
             while 1:
                 try:
                     nodes.extend(parser.parse(script))
-                except ninix.script.ParserError, e:
+                except ninix.script.ParserError as e:
                     done, script = e
                     nodes.extend(done)
                 else:
@@ -206,7 +196,7 @@ class SSTPRequestHandler(BaseSSTPRequestHandler):
                 if node[0] == ninix.script.SCRIPT_TAG and \
                    node[1] in self.PROHIBITED_TAGS:
                     self.send_response(400) # Bad Request
-                    self.log_error('Script: tag %s not allowed' % node[1])
+                    self.log_error('Script: tag {0} not allowed'.format(node[1]))
                     return 1
         return 0
 
@@ -219,6 +209,28 @@ class SSTPRequestHandler(BaseSSTPRequestHandler):
             return None
         return unicode(script, charset, 'replace')
 
+    def get_script_odict(self):
+        charset = self.headers.get('charset', 'Shift_JIS')
+        script_odict = OrderedDict()
+        if_ghost = None
+        for header in self.headers.headers:
+            line = unicode(header, charset, 'replace')
+            if not line.lower().startswith('script:'):
+                if line.lower().startswith('ifghost:'):
+                    if_ghost = line[8:].strip()
+                else:
+                    if_ghost = None
+                continue
+            script = line[7:].strip()
+            if self.check_script(script):
+                return
+            if if_ghost is None:
+                script_odict[''] = script
+            else:
+                script_odict[if_ghost] = script
+            if_ghost = None
+        return script_odict
+
     def get_script_if_ghost(self, current=0):
         charset = self.headers.get('charset', 'Shift_JIS')
         default = None
@@ -234,16 +246,19 @@ class SSTPRequestHandler(BaseSSTPRequestHandler):
                     continue
                 script = line[7:].strip()
                 if current: # NOTIFY
-                    ghost = self.server.callback['get_ghost_name']()
+                    ghost = self.server.request_parent('GET', 'get_ghost_name')
                     if ghost == if_ghost:
                         return script, if_ghost
                 else: # SEND
-                    if self.server.callback['if_ghost'](if_ghost):
+                    if self.server.request_parent('GET', 'if_ghost', if_ghost):
                         return script, if_ghost
                 if default is None:
                     default = script, if_ghost
         if default is None:
-            return self.get_script(), None
+            script = self.headers.get('script', None)
+            if script is not None:
+                script = unicode(script, charset, 'replace')
+            default = script, None
         return default
 
     def get_entry_db(self):
@@ -293,7 +308,7 @@ class SSTPRequestHandler(BaseSSTPRequestHandler):
             handle.connect(path)
         except socket.error:
             handle = None # discard socket object
-            sys.stderr.write('cannot open Unix socket: %s\n' % path)
+            logging.error('cannot open Unix socket: {0}'.format(path))
         if handle is None:
             self.send_response(400) # Bad Request
             self.log_error('Invalid HWnd: header field')
@@ -306,7 +321,7 @@ class SSTPRequestHandler(BaseSSTPRequestHandler):
             codecs.lookup(charset)
         except:
             self.send_response(420, 'Refuse (unsupported charset)')
-            self.log_error('Unsupported charset %s' % repr(charset))
+            self.log_error('Unsupported charset {0}'.format(repr(charset)))
         else:
             return 1
         return 0
@@ -342,15 +357,8 @@ class SSTPRequestHandler(BaseSSTPRequestHandler):
         if not self.local_request():
             host, port = self.client_address
             self.send_response(420)
-            self.log_error('Unauthorized EXECUTE/1.3 request from %s' % host)
-            return
-        self.handle_command()
-
-    def do_EXECUTE_1_5(self):
-        if not self.local_request():
-            host, port = self.client_address
-            self.send_response(420)
-            self.log_error('Unauthorized EXECUTE/1.5 request from %s' % host)
+            self.log_error(
+                'Unauthorized EXECUTE/1.3 request from {0}'.format(host))
             return
         self.handle_command()
 
@@ -361,36 +369,40 @@ class SSTPRequestHandler(BaseSSTPRequestHandler):
         if sender is None:
             return
         command = self.get_command()
+        charset = self.headers.get('charset', 'Shift_JIS')
         if command is None:
             return
         elif command == 'getname':
             self.send_response(200)
-            name = self.server.callback['get_ghost_name']()
-            self.wfile.write(''.join((name.encode('utf-8', 'ignore'), '\r\n')))
-            self.wfile.write('Charset: UTF-8\r\n')
+            name = self.server.request_parent('GET', 'get_ghost_name')
+            self.wfile.write(''.join((name.encode(charset, 'ignore'), '\r\n')))
+            self.wfile.write('\r\n')
         elif command == 'getversion':
             self.send_response(200)
-            self.wfile.write('ninix-aya %s\r\n' % ninix.version.VERSION)
+            self.wfile.write('ninix-aya {0}\r\n'.format(ninix.version.VERSION))
+            self.wfile.write('\r\n')
         elif command == 'quiet':
             self.send_response(200)
-            self.server.callback['keep_silence'](True)
+            self.server.request_parent('NOTIFY', 'keep_silence', True)
         elif command == 'restore':
             self.send_response(200)
-            self.server.callback['keep_silence'](False)
+            self.server.request_parent('NOTIFY', 'keep_silence', False)
         elif command == 'getnames':
             self.send_response(200)
-            for name in self.server.callback['get_ghost_names']():
+            for name in self.server.request_parent('GET', 'get_ghost_names'):
                 self.wfile.write(
-                    ''.join((name.encode('utf-8', 'ignore'), '\r\n')))
-            self.wfile.write('Charset: UTF-8\r\n')
+                    ''.join((name.encode(charset, 'ignore'), '\r\n')))
+            self.wfile.write('\r\n')
         elif command == 'checkqueue':
             self.send_response(200)
-            count, total = self.server.callback['check_request_queue'](sender)
+            count, total = self.server.request_parent(
+                'GET', 'check_request_queue', sender)
             self.wfile.write(''.join((count, '\r\n')))
             self.wfile.write(''.join((total, '\r\n')))
+            self.wfile.write('\r\n')
         else:
             self.send_response(501) # Not Implemented
-            self.log_error('Not Implemented (%s)' % command)
+            self.log_error('Not Implemented ({0})'.format(command))
 
     def get_command(self):
         charset = self.headers.get('charset', 'Shift_JIS')
@@ -411,8 +423,8 @@ class SSTPRequestHandler(BaseSSTPRequestHandler):
         if sentence is None:
             return
         self.send_response(200) # OK
-        self.server.callback['enqueue_event'](
-            'OnCommunicate', sender, sentence)
+        self.server.request_parent(
+            'NOTIFY', 'enqueue_event', 'OnCommunicate', sender, sentence)
         return
 
     def get_sentence(self):
