@@ -24,7 +24,6 @@ import gtk
 import cairo
 
 import ninix.seriko
-import ninix.menu
 import ninix.pix
 
 
@@ -35,7 +34,6 @@ class Surface(object):
 
     def __init__(self):
         self.window = []
-        self.__menu = None
         self.desc = None
         self.request_parent = lambda *a: None # dummy
         self.mikire = 0
@@ -62,8 +60,6 @@ class Surface(object):
         for surface_window in self.window:
             surface_window.destroy()
         self.window = []
-        ##self.__menu.finalize()
-        self.__menu = None
 
     def create_gtk_window(self, title, skip_taskbar):
         window = ninix.pix.TransparentWindow()
@@ -140,17 +136,13 @@ class Surface(object):
                 'NOTIFY', 'notify_event', 'OnKeyPress', name, keycode)
         return True
 
-    def window_stick(self):
-        stick = self.__menu.get_stick()
+    def window_stick(self, stick):
         for window in self.window:
             if stick:
                 window.window.stick()
             else:
                 window.window.unstick()
 
-    def open_popup_menu(self, button, side):
-        self.__menu.popup(button, side)
-
     re_surface_id = re.compile('^surface([0-9]+)$')
 
     def get_seriko(self, surface):
@@ -292,52 +284,24 @@ class Surface(object):
         self.__surface = surface
         self.add_window(0, '0', alias0, mayuna, bind['sakura'])
         self.add_window(1, '10', alias1, mayuna, bind['kero'])
-        self.__menu = ninix.menu.Menu()
-        self.__menu.set_responsible(self.handle_request)
+
+    def get_menu_pixmap(self):
         top_dir = self.prefix
         name = self.desc.get('menu.background.bitmap.filename',
                              'menu_background.png')
         name = name.replace('\\', '/')
-        path = os.path.join(top_dir, name)
-        if not os.path.exists(path):
-            top_dir = os.path.join(
-                self.request_parent('GET', 'get_prefix'), 'ghost', 'master')
-            path = os.path.join(top_dir, 'menu_background.png')
-            if os.path.exists(path):
-                path_background = path
-            else:
-                path_background = None
-            path = os.path.join(top_dir, 'menu_sidebar.png')
-            if os.path.exists(path):
-                path_sidebar = path
-            else:
-                path_sidebar = None
-            path = os.path.join(top_dir, 'menu_foreground.png')
-            if os.path.exists(path):
-                path_foreground = path
-            else:
-                path_foreground = None
-        else:
-            path_background = path
-            name = self.desc.get('menu.sidebar.bitmap.filename',
-                                 'menu_sidebar.png')
-            name = name.replace('\\', '/')
-            path = os.path.join(top_dir, name)
-            if os.path.exists(path):
-                path_sidebar = path
-            else:
-                path_sidebar = None
-            name = self.desc.get('menu.foreground.bitmap.filename',
-                                 'menu_foreground.png')
-            name = name.replace('\\', '/')
-            path = os.path.join(top_dir, name)
-            if os.path.exists(path):
-                path_foreground = path
-            else:
-                path_foreground = None
-        if path_background:
-            self.__menu.set_pixmap(
-                path_background, path_sidebar, path_foreground)
+        path_background = os.path.join(top_dir, name)
+        name = self.desc.get('menu.sidebar.bitmap.filename',
+                             'menu_sidebar.png')
+        name = name.replace('\\', '/')
+        path_sidebar = os.path.join(top_dir, name)
+        name = self.desc.get('menu.foreground.bitmap.filename',
+                             'menu_foreground.png')
+        name = name.replace('\\', '/')
+        path_foreground = os.path.join(top_dir, name)
+        return path_background, path_sidebar, path_foreground
+
+    def get_menu_fontcolor(self):
         fontcolor_r = self.desc.get_with_type('menu.background.font.color.r', int, 0)
         fontcolor_g = self.desc.get_with_type('menu.background.font.color.g', int, 0)
         fontcolor_b = self.desc.get_with_type('menu.background.font.color.b', int, 0)
@@ -346,8 +310,7 @@ class Surface(object):
         fontcolor_g = self.desc.get_with_type('menu.foreground.font.color.g', int, 0)
         fontcolor_b = self.desc.get_with_type('menu.foreground.font.color.b', int, 0)
         foreground = (fontcolor_r, fontcolor_g, fontcolor_b)
-        self.__menu.set_fontcolor(background, foreground)
-        self.__menu.create_mayuna_menu(self.get_mayuna_menu())
+        return background, foreground
 
     def add_window(self, side, default, alias=None, mayuna={}, bind={}):
         assert len(self.window) == side
