@@ -27,6 +27,7 @@ import cStringIO
 import imp
 import locale
 import multiprocessing
+from collections import OrderedDict
 
 if os.name == 'nt': # locale setting for windows
     lang = os.getenv('LANG')
@@ -51,6 +52,9 @@ import ninix.install
 import ninix.plugin
 import ninix.nekodorif
 import ninix.kinoko
+import ninix.menu
+from ninix.metamagic import Holon
+from ninix.metamagic import Meme
 
 USAGE = 'Usage: ninix [options]'
 parser = optparse.OptionParser(USAGE)
@@ -220,39 +224,39 @@ class SSTPControler(object):
             entry_db, request_handler = self.__sstp_queue.pop(0)
         working = bool(event is not None)
         for if_ghost in script_odict.keys():
-           if if_ghost and self.request_parent('GET', 'if_ghost', if_ghost, working):
-              self.request_parent('NOTIFY', 'select_current_sakura', if_ghost)
-              default_script = script_odict[if_ghost]
-              break
+            if if_ghost and self.request_parent('GET', 'if_ghost', if_ghost, working):
+                self.request_parent('NOTIFY', 'select_current_sakura', if_ghost)
+                default_script = script_odict[if_ghost]
+                break
         else:
-           if not self.request_parent('GET', 'get_preference', 'allowembryo'):
-              if event is None:
-                 if request_handler:
-                    request_handler.send_response(420) # Refuse
-                 return
-              else:
-                 default_script = None
-           else:
-              if '' in script_odict: # XXX
-                 default_script = script_odict['']
-              else:
-                 default_script = script_odict.values()[0]
+            if not self.request_parent('GET', 'get_preference', 'allowembryo'):
+                if event is None:
+                    if request_handler:
+                        request_handler.send_response(420) # Refuse
+                    return
+                else:
+                    default_script = None
+            else:
+                if '' in script_odict: # XXX
+                    default_script = script_odict['']
+                else:
+                    default_script = script_odict.values()[0]
         if event is not None:
-           script = self.request_parent('GET', 'get_event_response', event)
+            script = self.request_parent('GET', 'get_event_response', event)
         else:
-           script = None
+            script = None
         if not script:
-              script = default_script
+            script = default_script
         if script is None:
-           if request_handler:
-              request_handler.send_response(204) # No Content
-           return
+            if request_handler:
+                request_handler.send_response(204) # No Content
+            return
         self.set_sstp_flag(sender)
         self.request_parent(
-           'NOTIFY', 'enqueue_script',
-           script, sender, handle, address,
-           show_sstp_marker, use_translator, entry_db,
-           request_handler, temp_mode=True)
+            'NOTIFY', 'enqueue_script',
+            script, sender, handle, address,
+            show_sstp_marker, use_translator, entry_db,
+            request_handler, temp_mode=True)
 
     def receive_sstp_request(self):
         try:
@@ -413,6 +417,45 @@ class PluginControler:
         return None
 
 
+class BalloonMeme(Meme):
+
+    def __init__(self, key):
+        Meme.__init__(self, key)
+        self.request_parent = lambda *a: None # dummy
+
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
+    def create_menuitem(self, data):
+        desc, balloon = data
+        subdir = balloon['balloon_dir'][0]
+        name = desc.get('name', subdir)
+        return self.request_parent(
+            'GET', 'create_balloon_menuitem', name, self.key)
+
+    def delete_by_myself(self):
+        self.request_parent('NOTIFY', 'delete_balloon', self.key)
+
+
+class Ghost(Holon):
+
+    def __init__(self, key):
+        Holon.__init__(self, key)
+        self.request_parent = lambda *a: None # dummy
+        
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
+    def create_menuitem(self, data):
+        return self.request_parent('GET', 'create_menuitem', self.key, data)
+
+    def delete_by_myself(self):
+        self.request_parent('NOTIFY', 'delete_ghost', self.key)
+
+    def create_instance(self, data):
+        return self.request_parent('GET', 'create_ghost', data)
+
+
 class Application(object):
 
     def __init__(self, sstp_port=[9801, 11000]):
@@ -436,15 +479,29 @@ class Application(object):
         self.current_sakura = None
         # create installer
         self.installer = ninix.install.Installer()
-        self.ghosts = ninix.home.search_ghosts()
-        self.balloons = ninix.home.search_balloons()
+        # create popup menu
+        self.__menu = ninix.menu.Menu()
+        self.__menu.set_responsible(self.handle_request)
+        self.__menu_owner = None
+        self.ghosts = OrderedDict()
+        odict_baseinfo = ninix.home.search_ghosts()
+        for key, value in odict_baseinfo.items():
+            holon = Ghost(key)
+            holon.set_responsible(self.handle_request)
+            self.ghosts[key] = holon 
+            holon.baseinfo = value
+        self.balloons = OrderedDict()
+        odict_baseinfo = ninix.home.search_balloons()
+        for key, value in odict_baseinfo.items():
+            meme = BalloonMeme(key)
+            meme.set_responsible(self.handle_request)
+            self.balloons[key] = meme
+            meme.baseinfo = value
+        self.balloon_menu = self.create_balloon_menu()
         self.plugins = ninix.home.search_plugins()
         self.nekoninni = ninix.home.search_nekoninni()
         self.katochan = ninix.home.search_katochan()
         self.kinoko = ninix.home.search_kinoko()
-        self.ghost_list = [
-            self.__create_ghost_list_item(i) for i in \
-                sorted(self.ghosts.keys())]
 
     def handle_request(self, event_type, event, *arglist, **argdict):
         assert event_type in ['GET', 'NOTIFY']
@@ -455,9 +512,8 @@ class Application(object):
             'get_otherghostname': self.communicate.get_otherghostname,
             'rebuild_ghostdb': self.communicate.rebuild_ghostdb,
             'reset_sstp_flag': self.sstp_controler.reset_sstp_flag,
-            'start_sakura': self.start_sakura_cb,
             'send_message': self.communicate.send_message,
-            'get_sstp_port' : self.sstp_controler.get_sstp_port,
+            'get_sstp_port': self.sstp_controler.get_sstp_port,
             }
         handler = handlers.get(event,
                                getattr(self, event,
@@ -476,9 +532,9 @@ class Application(object):
             if filetype == 'ghost':
                 self.add_sakura(target_dir)
             elif filetype == 'supplement':
-                pass ## FIXME
+                self.add_sakura(target_dir) # XXX: reload
             elif filetype == 'balloon':
-                self.balloons = ninix.home.search_balloons()
+                self.add_balloon(target_dir)
             elif filetype == 'plugin':
                 self.plugins = ninix.home.search_plugins()
             elif filetype == 'nekoninni':
@@ -490,17 +546,12 @@ class Application(object):
 
     @property
     def current_sakura_instance(self):
-        set_type, i, j = self.current_sakura
-        num = self.get_index(i) ## FIXME
-        return self.ghost_list[num]['instance']
-
-    def get_index(self, i): ## FIXME
-        keys = sorted(self.ghosts.keys())
-        return keys.index(i)
+        return self.ghosts[self.current_sakura].instance
 
     def create_ghost(self, data):
-        ghost = ninix.sakura.Sakura(data)
+        ghost = ninix.sakura.Sakura()
         ghost.set_responsible(self.handle_request)
+        ghost.new(*data)
         return ghost
 
     def get_sakura_cantalk(self):
@@ -514,7 +565,7 @@ class Application(object):
 
     def get_ghost_name(self): ## FIXME
         sakura = self.current_sakura_instance
-        return '{0},{1}'.format(sakura.get_selfname(), sakura.get_keroname())
+        return sakura.get_ifghost()
 
     def enqueue_event(self, event, *arglist, **argdict):
         self.current_sakura_instance.enqueue_event(event, *arglist, **argdict)
@@ -530,62 +581,134 @@ class Application(object):
                               db, request_handler)
 
     def get_working_ghost(self, cantalk=0):
-        working_list = [item['instance'] for item in self.ghost_list \
-                            if item is not None and \
-                            item['instance'] is not None and \
-                            item['instance'].is_running()]
-        if cantalk:
-            working_list = [sakura for sakura in working_list if sakura.cantalk]
-        return working_list
+        for value in self.ghosts.values():
+            sakura = value.instance
+            if sakura is None:
+                continue
+            if not sakura.is_running():
+                continue
+            if cantalk and not sakura.cantalk:
+                continue
+            yield sakura
 
-    def get_ghost_list(self): ## FIXME
-        return self.ghost_list
+    def getstring(self, name):
+        return self.__menu_owner.getstring(name)
 
-    def __create_ghost_list_item(self, i):
-        assert i in self.ghosts
-        if self.ghosts[i] is None:
-            return None
-        set_type = 'g'
-        desc = self.ghosts[i][0]
-        shiori_dir = self.ghosts[i][1]
+    def stick_window(self):
+        stick = self.__menu.get_stick()
+        self.__menu_owner.stick_window(stick)
+
+    def toggle_bind(self, args):
+        self.__menu_owner.toggle_bind(args)
+
+    def select_shell(self, key):
+        self.__menu_owner.select_shell(key)
+
+    def select_balloon(self, key):
+        desc, balloon = self.get_balloon_description(key)
+        self.__menu_owner.select_balloon(key, desc, balloon)
+
+    def get_current_balloon_directory(self): ## FIXME
+        return self.__menu_owner.get_current_balloon_directory()
+
+    def start_sakura_cb(self, key): ## FIXME
+        self.start_sakura(key, init=1) # XXX
+
+    def select_sakura(self, key):
+        if self.__menu_owner.busy():
+            gtk.gdk.beep()
+            return
+        self.change_sakura(self.__menu_owner, key, 'manual')
+
+    def notify_site_selection(self, args):
+        self.__menu_owner.notify_site_selection(args)
+
+    def close_sakura(self):
+        self.__menu_owner.close()
+
+    def about(self):
+        self.__menu_owner.about()
+
+    def vanish(self):
+        self.__menu_owner.vanish()
+
+    def network_update(self):
+        self.__menu_owner.network_update()
+
+    def open_popup_menu(self, sakura, button, side):
+        self.__menu_owner = sakura
+        path_background, path_sidebar, path_foreground = \
+            self.__menu_owner.get_menu_pixmap()
+        self.__menu.set_pixmap(path_background, path_sidebar, path_foreground)
+        background, foreground = self.__menu_owner.get_menu_fontcolor()
+        self.__menu.set_fontcolor(background, foreground)
+        mayuna_menu = self.__menu_owner.get_mayuna_menu()
+        self.__menu.create_mayuna_menu(mayuna_menu)
+        self.__menu.popup(button, side)
+
+    def get_ghost_menus(self):
+        for value in self.ghosts.values():
+            yield value.menuitem
+
+    def get_shell_menu(self):
+        return self.__menu_owner.get_shell_menu()
+
+    def get_balloon_menu(self):
+        current_key = self.get_current_balloon_directory()
+        for key in self.balloons:
+            menuitem = self.balloons[key].menuitem
+            menuitem.set_sensitive(key != current_key) # not working
+        return self.balloon_menu
+
+    def create_balloon_menuitem(self, balloon_name, balloon_key):
+        return self.__menu.create_meme_menuitem(
+            balloon_name, balloon_key, self.select_balloon)
+
+    def create_balloon_menu(self):
+        balloon_menuitems = OrderedDict()
+        for key in self.balloons.keys():
+            balloon_menuitems[key] = self.balloons[key].menuitem
+        return self.__menu.create_meme_menu(balloon_menuitems)
+
+    def create_shell_menu(self, menuitems):
+        return self.__menu.create_meme_menu(menuitems)
+
+    def create_shell_menuitem(self, shell_name, shell_key):
+        return self.__menu.create_meme_menuitem(
+            shell_name, shell_key, self.select_shell)
+
+    def create_menuitem(self, key, baseinfo):
+        desc = baseinfo[0]
+        shiori_dir = baseinfo[1]
         icon = desc.get('icon', None)
         if icon is not None:
+            if os.name == 'nt':
+                # XXX: path should be encoded in mbcs on Windows
+                icon = icon.encode('mbcs')
             icon_path = os.path.join(shiori_dir, icon)
             if not os.path.exists(icon_path):
                 icon_path = None
         else:
             icon_path = None
-        name = desc.get('name', i)
-        shell_list = []
-        surface_set = self.ghosts[i][3]
-        for j in sorted(surface_set.keys()):
-            value = (set_type, i, j)
-            shell_name = surface_set[j][0]
-            shell_list.append((shell_name, value))
-        if not shell_list: # XXX
-            return None
-        item = {}
-        item['name'] = name
-        item['icon'] = icon_path
-        item['shell'] = shell_list
-        item['instance'] = None
-        return item
+        name = desc.get('name')
+        start_menuitem = self.__menu.create_ghost_menuitem(
+            name, icon_path, key, self.start_sakura_cb) # XXX
+        select_menuitem = self.__menu.create_ghost_menuitem(
+            name, icon_path, key, self.select_sakura)
+        menuitem = {
+            'Summon': start_menuitem,
+            'Change': select_menuitem,
+            }
+        return menuitem
 
-    def update_ghost_list(self, i, sakura):
-        keys = sorted(self.ghosts.keys())
-        num = keys.index(i)
-        self.ghost_list[num] = self.__create_ghost_list_item(i)
-        if self.ghost_list[num] is not None:
-            self.ghost_list[num]['instance'] = sakura
-        else:
-            pass ## FIXME: ghost instance should be deleted
+    def delete_ghost(self, key):
+        assert key in self.ghosts
+        del self.ghosts[key]
 
-    def get_balloon_list(self):
+    def get_balloon_list(self): ## FIXME
         balloon_list = []
-        balloons = self.balloons
-        keys = sorted(balloons.keys())
-        for key in keys:
-            desc, balloon = balloons[key]
+        for key in self.balloons.keys():
+            desc, balloon = self.balloons[key].baseinfo
             subdir = balloon['balloon_dir'][0]
             name = desc.get('name', subdir)
             balloon_list.append((name, subdir))
@@ -631,148 +754,121 @@ class Application(object):
         # choose default ghost/shell
         directory = self.prefs.get('sakura_dir')
         name = self.prefs.get('sakura_name') # XXX: backward compat
-        surface = self.prefs.get('sakura_surface')
-        default_sakura = self.find_ghost_by_dir(directory, surface) or \
-                         self.find_ghost_by_name(name, surface) or \
+        default_sakura = self.find_ghost_by_dir(directory) or \
+                         self.find_ghost_by_name(name) or \
                          self.choose_default_sakura()
         # load ghost
         self.current_sakura = default_sakura
-        keys = sorted(self.ghosts.keys())
-        for num, i in enumerate(keys):
-            if self.ghost_list[num] is None: # XXX
-                continue
-            sakura = self.create_ghost(self.ghosts[i])
-            self.ghost_list[num]['instance'] = sakura
-        ##names = self.get_ghost_names()
-        ##for i in range(len(names)):
+        ##for i, name in enumerate(self.get_ghost_names()):
         ##    logging.info(
         ##        'GHOST({0:d}): {1}'.format(
-        ##            i, names[i].encode('utf-8', 'ignore')))
+        ##            i, name.encode('utf-8', 'ignore')))
         self.start_sakura(self.current_sakura, init=1)
 
-    def find_ghost_by_dir(self, directory, surface):
-        if directory in self.ghosts:
-            surface_set = self.ghosts[directory][3]
-            if surface not in surface_set:
-                surface = sorted(surface_set.keys())[0]
-            return ('g', directory, surface)
-        return None
+    def find_ghost_by_dir(self, directory):
+        return directory if directory in self.ghosts else None
 
-    def find_ghost_by_name(self, name, surface):
-        for i in self.ghosts:
-            if self.ghosts[i] is None:
-                continue
-            desc = self.ghosts[i][0]
+    def find_ghost_by_name(self, name):
+        for key in self.ghosts:
+            sakura = self.ghosts[key].instance
             try:
-                if desc.get('name') == name:
-                    surface_set = self.ghosts[i][3]
-                    assert surface_set
-                    if surface not in surface_set:
-                        surface = sorted(surface_set.keys())[0]
-                    return ('g', i, surface)
+                if sakura.get_name(default=None) == name:
+                    return key
             except: # old preferences(EUC-JP)
                 pass
         return None
 
     def choose_default_sakura(self):
-        keys = sorted(self.ghosts.keys())
-        for i in keys:
-            if self.ghosts[i] is None:
-                continue
-            if self.ghosts[i][3]: # surface_set
-                surface = sorted(self.ghosts[i][3].keys())
-                return ('g', i, surface[0])
-        raise RuntimeError, 'should not reach here'
+        return list(self.ghosts.keys())[0]
 
     def find_balloon_by_name(self, name):
-        balloons = self.balloons
-        for desc, balloon in balloons.values(): # FIXME: sort
+        for key in self.balloons:
+            desc, balloon = self.balloons[key].baseinfo
             try:
                 if desc.get('name') == name:
-                    return desc, balloon
+                    return key
                 if balloon['balloon_dir'][0] == ninix.home.get_normalized_path(name): # XXX
-                    return desc, balloon
+                    return key
             except: # old preferences(EUC-JP)
                 pass
         return None
 
     def find_balloon_by_subdir(self, subdir):
-        balloons = self.balloons
-        for desc, balloon in balloons.values(): ## FIXME: sort
+        for key in self.balloons:
+            desc, balloon = self.balloons[key].baseinfo
             try:
                 if balloon['balloon_dir'][0] == subdir:
-                    return desc, balloon
+                    return key
                 if ninix.home.get_normalized_path(desc.get('name')) == subdir: # XXX
-                    return desc, balloon
+                    return key
             except: # old preferences(EUC-JP)
                 pass
         return None
 
     def run(self):
-        self.timeout_id = glib.timeout_add(100, self.do_idle_tasks) # 100ms
+        self.timeout_id = glib.timeout_add(100, self.do_idle_tasks) # 100[ms]
         gtk.main()
 
-    def get_ghost_names(self): ## FIXME
-        return [item['instance'].get_selfname() for item in self.ghost_list if item is not None]
+    def get_ghost_names(self):
+        for value in self.ghosts.values():
+            if value.instance is not None:
+                yield value.instance.get_selfname() ## FIXME
 
     def if_ghost(self, if_ghost, working=False):
-        for sakura in (item['instance'] for item in self.ghost_list \
-                           if item is not None):
-            assert sakura is not None
+        for sakura in (value.instance for value in self.ghosts.values() \
+                           if value.instance is not None):
             if working:
                if not sakura.is_running() or not sakura.cantalk:
                   continue
-            ghost_name = '{0},{1}'.format(sakura.get_selfname(),
-                                          sakura.get_keroname())
-            if ghost_name == if_ghost:
+            if sakura.ifghost(if_ghost):
                return 1
         else:
             return 0
 
     def update_sakura(self, name, sender):
-        item = self.find_ghost_by_name(name, 0)
-        if item is None:
+        key = self.find_ghost_by_name(name, '')
+        if key is None:
             return
-        sakura = self.select_ghost_from_list(item)
+        sakura = self.ghosts[key].instance
         if not sakura.is_running():
-            self.start_sakura(item, init=1)
+            self.start_sakura(key, init=1)
         sakura.enqueue_script('\![updatebymyself]\e', sender,
                               None, None, 0, 0, None)
 
     def select_current_sakura(self, ifghost=None):
         if ifghost is not None:
-            for item in self.ghost_list:
-                if item is None:
+            for value in self.ghosts.values():
+                sakura = value.instance
+                if sakura is None:
                     continue
-                sakura = item['instance']
-                assert sakura is not None
-                names = '{0},{1}'.format(sakura.get_selfname(),
-                                         sakura.get_keroname())
-                name =  '{0}'.format(sakura.get_selfname())
-                if ifghost in [name, names]:
+                if sakura.ifghost(ifghost):
                     if not sakura.is_running():
-                        self.current_sakura = item['shell'][0][1] ## FIXME
+                        self.current_sakura = value.key
                         self.start_sakura(self.current_sakura, init=1, temp=1) ## FIXME
                     else:
-                        self.current_sakura = sakura.current
+                        self.current_sakura = sakura.key
                     break
                 else:
                     pass
             else:
                 return
         else:
-            working_list = self.get_working_ghost(cantalk=1)
+            working_list = list(self.get_working_ghost(cantalk=1))
             if working_list:
-                self.current_sakura = random.choice(working_list).current
+                self.current_sakura = random.choice(working_list).key
             else:
                 return ## FIXME
 
+    def set_menu_sensitive(self, key, flag):
+        menuitems = self.ghosts[key].menuitem
+        for item in menuitems.values():
+            item.set_sensitive(flag)
+
     def close_ghost(self, sakura):
-        if not self.get_working_ghost():
-            set_type, i, j = sakura.current
-            self.prefs.set_current_sakura(i, j)
+        if not any(self.get_working_ghost()):
+            self.prefs.set_current_sakura(sakura.key)
             self.quit()
-        elif self.current_sakura == sakura.current:
+        elif self.current_sakura == sakura.key:
             self.select_current_sakura()
 
     def close_all_ghosts(self):
@@ -796,153 +892,145 @@ class Application(object):
             pass ## FIXME
 
     def select_ghost(self, sakura, sequential, event=1, vanished=0):
-        keys = sorted(self.ghosts.keys())
-        ghosts = [key for key in keys if self.ghosts[key] is not None]
-        if len(ghosts) < 2:
+        keys = list(self.ghosts.keys())
+        if len(keys) < 2:
             return
         # select another ghost
-        set_type, i, j = sakura.current
-        assert set_type == 'g'
         if sequential:
-            i = (keys.index(i) + 1) % len(keys)
+            key = (keys.index(sakura.key) + 1) % len(keys)
         else:
-            ghosts.remove(i)
-            i = random.choice(ghosts)
-        surface = sorted(self.ghosts[i][3].keys())
-        if self.current_sakura == sakura.current: # XXX
-            self.current_sakura = ('g', i, surface[0])
-        self.change_sakura(sakura, ('g', i, surface[0]), 'automatic',
-                           event, vanished)
+            keys.remove(sakura.key)
+            key = random.choice(keys)
+        self.change_sakura(sakura, key, 'automatic', event, vanished)
 
     def select_ghost_by_name(self, sakura, name, event=1):
-        item = self.find_ghost_by_name(name, 0)
-        if item is None:
+        key = self.find_ghost_by_name(name, 0)
+        if key is None:
             return
-        self.change_sakura(sakura, item, 'automatic', event)
+        self.change_sakura(sakura, key, 'automatic', event)
 
-    def change_sakura(self, sakura, item, method, event=1, vanished=0):
-        set_type, i, j = item
-        assert self.ghosts[i] is not None
-        if sakura.current == item: # XXX: needs reloading?
+    def change_sakura(self, sakura, key, method, event=1, vanished=0):
+        if sakura.key == key: # XXX: needs reloading?
             return
-        if sakura.current[1] == i:
-            return # XXX: not shell change
-        assert set_type == 'g'
-        desc, shiori_dir, use_makoto, surface_set, prefix, \
-            shiori_dll, shiori_name = self.ghosts[i]
-        assert surface_set and j in surface_set
-        name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
-            surface_set[j]
-        def proc(self=self, item=item):
-            self.stop_sakura(sakura, self.start_sakura, item, sakura.current)
+        def proc(self=self, key=key):
+            self.stop_sakura(sakura, self.start_sakura, key, sakura.key)
         if vanished:
             sakura.finalize()
-            self.start_sakura(item, sakura.current, vanished)
+            self.start_sakura(key, sakura.key, vanished)
             self.close_ghost(sakura)
         elif not event:
             proc()
         else:
-            name = surface_desc.get('sakura.name', desc.get('sakura.name'))
+            name = self.ghosts[key].instance.get_selfname(default='')
             sakura.enqueue_event('OnGhostChanging', name, method, proc=proc)
 
     def stop_sakura(self, sakura, starter=None, *args):
         sakura.finalize()
         if starter is not None:
             starter(*args)
+        self.set_menu_sensitive(sakura.key, True)
         self.close_ghost(sakura)
 
-    def select_ghost_from_list(self, item):
-        set_type, i, j = item
-        assert set_type == 'g'
-        assert self.ghosts[i] is not None
-        num = self.get_index(i) ## FIXME
-        assert self.ghost_list[num]['instance'] is not None
-        return self.ghost_list[num]['instance']
-
-    def start_sakura(self, item, prev=None, vanished=0, init=0, temp=0):
-        set_type, i, j = item
-        assert set_type == 'g'
-        sakura = self.select_ghost_from_list(item)
-        keys = sorted(self.ghosts.keys())
+    def start_sakura(self, key, prev=None, vanished=0, init=0, temp=0):
+        sakura = self.ghosts[key].instance
+        assert sakura is not None
         if prev is not None:
-            prev_num = keys.index(prev[1])
-            assert self.ghosts[prev[1]] is not None
-            assert self.ghost_list[prev_num]['instance'] is not None
+            assert prev in self.ghosts ## FIXME: vanish case?
+            assert self.ghosts[prev].instance is not None
         if init:
             ghost_changed = 0
         else:
             assert prev is not None ## FIXME
-            if prev[0] == set_type and prev[1] == i:
+            if prev == key:
                 ghost_changed = 0
             else:
                 ghost_changed = 1
         if ghost_changed:
-            name = self.ghost_list[prev_num]['instance'].get_selfname()
+            name = self.ghosts[prev].instance.get_selfname()
         else:
             name = None
         sakura.notify_preference_changed()
-        sakura.start(item, init, temp, vanished, ghost_changed, name)
+        sakura.start(key, init, temp, vanished, ghost_changed, name)
+        self.set_menu_sensitive(key, False)
 
     def notify_preference_changed(self):
         for sakura in self.get_working_ghost():
             sakura.notify_preference_changed()
 
-    def start_sakura_cb(self, item):
-        self.start_sakura(item, init=1)
-
-    def get_balloon_description(self, subdir):
-        balloon = self.find_balloon_by_subdir(subdir)
-        if balloon is None:
+    def get_balloon_description(self, subdir): ## FIXME
+        key = self.find_balloon_by_subdir(subdir)
+        if key is None:
             ##logging.warning('Balloon {0} not found.'.format(name))
             default_balloon = self.prefs.get('default_balloon')
-            balloon = self.find_balloon_by_subdir(default_balloon)
-        if balloon is None:
-            balloons = self.balloons
-            assert balloons
-            key = sorted(balloons.keys())[0]
-            balloon = balloons[key]
-        return balloon
+            key = self.find_balloon_by_subdir(default_balloon)
+        if key is None:
+            key = self.balloons.keys()[0]
+        return self.balloons[key].baseinfo
 
     def reload_current_sakura(self, sakura):
         self.save_preferences()
-        set_type, i, j = sakura.current
-        assert set_type == 'g'
+        key = sakura.key
         ghost_dir = os.path.split(sakura.get_prefix())[1] # XXX
         ghost_conf = ninix.home.search_ghosts([ghost_dir])
         if ghost_conf:
-            self.ghosts[i] = ghost_conf[i]
-            sakura.new(*ghost_conf[i]) # reset
+            self.ghosts[key].baseinfo = ghost_conf[key]
         else:
-            self.ghosts[i] = None ## FIXME
-        self.update_ghost_list(i, sakura)
-        item = (set_type, i, j)
-        self.start_sakura(item, item, init=1) 
+            self.close_ghost(sakura) ## FIXME
+            del self.ghosts[key]
+            return ## FIXME
+        self.start_sakura(key, key, init=1) 
 
     def add_sakura(self, ghost_dir):
         if ghost_dir in self.ghosts:
             exists = 1
             logging.warning('INSTALLED GHOST CHANGED: {0}'.format(ghost_dir))
-            ## FIXME: reload if working
         else:
             exists = 0
             logging.info('NEW GHOST INSTALLED: {0}'.format(ghost_dir))
-        keys = list(self.ghosts.keys()) ## FIXME
-        if not exists:
-            keys.append(ghost_dir)
-        keys.sort() ## FIXME
-        num = keys.index(ghost_dir)
         ghost_conf = ninix.home.search_ghosts([ghost_dir])
         if ghost_conf:
-            self.ghosts[ghost_dir] = ghost_conf[ghost_dir]
+            if exists:
+                sakura = self.ghosts[ghost_dir].instance
+                if sakura.is_running(): # restart if working
+                    key = sakura.key
+                    def proc(self=self):
+                        self.ghosts[ghost_dir].baseinfo = ghost_conf[ghost_dir]
+                        logging.info('restarting....')
+                        self.start_sakura(key, key, init=1)
+                        logging.info('done.')
+                    self.stop_sakura(sakura, proc)
+            else:
+                holon = Ghost(ghost_dir)
+                holon.set_responsible(self.handle_request)
+                self.ghosts[ghost_dir] = holon
+                holon.baseinfo = ghost_conf[ghost_dir]
         else:
-            self.ghosts[ghost_dir] = None ## FIXME
-        item = self.__create_ghost_list_item(ghost_dir)
-        if item is not None: # XXX
-            item['instance'] = self.create_ghost(ghost_conf[ghost_dir])
-        if exists:
-            self.ghost_list[num] = item
+            if exists:
+                sakura = self.ghosts[ghost_dir].instance
+                if sakura.is_running(): # stop if working
+                    self.stop_sakura(sakura)
+                del self.ghosts[ghost_dir]
+
+    def add_balloon(self, balloon_dir):
+        if balloon_dir in self.balloons:
+            exists = 1
+            logging.warning('INSTALLED BALLOON CHANGED: {0}'.format(balloon_dir))
         else:
-            self.ghost_list.insert(num, item)
+            exists = 0
+            logging.info('NEW BALLOON INSTALLED: {0}'.format(balloon_dir))
+        balloon_conf = ninix.home.search_balloons([balloon_dir])
+        if balloon_conf:
+            if exists:
+                self.balloons[balloon_dir].baseinfo = balloon_conf[balloon_dir]
+            else:
+                meme = BalloonMeme(balloon_dir)
+                meme.set_responsible(self.handle_request)
+                self.balloons[balloon_dir] = meme
+                meme.baseinfo = balloon_conf[balloon_dir]
+        else:
+            if exists:
+                del self.balloons[balloon_dir]
+        self.balloon_menu = self.create_balloon_menu()
 
     def vanish_sakura(self, sakura):
         # remove ghost
@@ -961,28 +1049,27 @@ class Application(object):
                 except:
                     logging.error(
                         '*** REMOVE FAILED *** : {0}'.format(filename))
-        set_type, i, j = sakura.current
-        assert set_type == 'g'
         self.select_ghost(sakura, 0, vanished=1)
-        self.ghosts[i] = None
-        self.update_ghost_list(i, sakura)
+        del self.ghosts[sakura.key]
 
-    def select_plugin(self, item, target): ## FIXME
+    def select_plugin(self, item):
+        target = self.__menu_owner
         i, j = item
         plugin_name, plugin_dir, startup, menu_items = self.plugins[i]
         label, argv = menu_items[j]
         caller = {}
         caller['name'] = target.get_name()
         caller['directory'] = target.get_prefix()
-        caller['ifghost'] = '{0},{1}'.format(target.get_selfname(),
-                                             target.get_keroname())
+        caller['ifghost'] = target.get_ifghost()
         self.plugin_controler.exec_plugin(plugin_dir, argv, caller)
 
-    def select_nekodorif(self, nekodorif_dir, target):
+    def select_nekodorif(self, nekodorif_dir):
+        target = self.__menu_owner
         ninix.nekodorif.Nekoninni().load(nekodorif_dir,
                                          self.katochan, target)
 
-    def select_kinoko(self, data, target):
+    def select_kinoko(self, data):
+        target = self.__menu_owner
         ninix.kinoko.Kinoko(self.kinoko).load(data, target)
 
     def open_console(self):
@@ -995,13 +1082,11 @@ class Application(object):
         for sakura in self.get_working_ghost():
             sakura.save_history()
         history = {}
-        for i in self.ghosts:
-            if self.ghosts[i] is None:
-                continue
-            desc = self.ghosts[i][0]
-            name = desc.get('name', i)
+        for key in self.ghosts:
+            sakura = self.ghosts[key].instance
+            name = sakura.get_name(default=i)
             ghost_time = 0
-            prefix = self.ghosts[i][4]
+            prefix = sakura.get_prefix()
             path = os.path.join(prefix, 'HISTORY')
             if os.path.exists(path):
                 try:
@@ -1029,8 +1114,8 @@ class Application(object):
         self.usage_dialog.open(history)
 
     def search_ghosts(self): ## FIXME
-        balloons = self.get_balloon_list()
-        ghosts = [1 for item in self.ghost_list if item is not None]
+        balloons = self.balloons ## FIXME
+        ghosts = self.ghosts ## FIXME
         if len(ghosts) > 0 and len(balloons) > 0:
             self.confirmed = True
         return len(ghosts), len(balloons)
